#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <EzGraph.h>
#include "config.h"
#include "draw.h"
#include "obj_draw.h"
#include "obj.h"
#include "stage.h"
#include "emap.h"
#include "layout.h"
#include "uty.h"
#include "game.h"

static obj_t objs[OBJ_NUM_MAX];
static obj_t *obj_alive_list[oc_end];
static obj_t *obj_free_list;

obj_t *pop_obj_list(obj_t **pop_list);
void push_obj_list(obj_t *obj, obj_t **push_list);

static state_req_entry_t req_queue[OBJ_STATE_REQ_MAX];
static int req_num = 0;


obj_info_t obj_info[] = {
    {ot_wall, oc_static, 
     {create_wall, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_goal, oc_hard_dynamic, 
     {create_goal, act_goal, draw_goal, NULL, NULL, NULL, NULL}},
    {ot_stairs_l, oc_static,
     {create_stairs_l, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_stairs_r, oc_static,
     {create_stairs_r, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_slope_l, oc_static,
     {create_slope_l, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_slope_r, oc_static,
     {create_slope_r, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sign, oc_static, 
     {create_sign, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sting, oc_static,    // createot_sting_lʤɤˤʤΤdraw
     {create_sting, NULL, NULL, NULL, NULL, NULL, NULL}},
    {ot_sting_l, oc_static,  // ʲ4ot_stingȤƺΤcreate
     {NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sting_r, oc_static,
     {NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sting_u, oc_static,
     {NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sting_d, oc_static,
     {NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},

    {ot_chee, oc_soft_dynamic,
     {create_chee, act_chee, draw_chee, NULL, NULL, NULL, NULL}},


    {ot_fwall, oc_hard_dynamic,
     {create_fwall, act_fwall, draw_motion, NULL, NULL, NULL, NULL}},
    {ot_iwall, oc_hard_dynamic,
     {create_iwall, act_iwall, draw_motion, NULL, NULL, NULL, NULL}},
    {ot_barrier, oc_hard_dynamic,
     {create_barrier, act_barrier, draw_barrier, NULL, NULL, NULL, NULL}},



    {ot_gondin, oc_soft_dynamic,
//     {create_gondin, act_gondin, draw_static, NULL, NULL, NULL, NULL}},
     {create_gondin, act_creature, draw_motion, NULL, 
	 set_action_gondin, set_motion_gondin, set_emap_gondin}},
    {ot_hatopo, oc_soft_dynamic,
     //     {create_hatopo, act_hatopo, draw_hatopo}},
     {create_hatopo, act_creature, draw_motion, NULL, 
      set_action_hatopo, set_motion_hatopo, NULL}},
    {ot_gagga, oc_soft_dynamic,
     {create_gagga, act_creature, draw_gagga, post_draw_gagga,
      set_action_gagga, set_motion_gagga, set_emap_gagga}},

    {ot_nya, oc_hard_dynamic,
	{create_nya, NULL, draw_nya, NULL, NULL, NULL, NULL}},
//      set_action_nya, set_motion_nya}},
    {ot_punch, oc_hard_dynamic,
     {create_punch, act_punch, draw_punch,  NULL,NULL, NULL, NULL}},
    {ot_yaarashi, oc_soft_dynamic,
     {create_yaarashi, act_creature, draw_motion,  NULL,
      set_action_yaarashi, set_motion_yaarashi, set_emap_yaarashi}},
    {ot_ojigiri, oc_soft_dynamic,
     {create_ojigiri, act_creature, NULL, post_draw_ojigiri,
      set_action_ojigiri, set_motion_ojigiri, NULL}},
    {ot_sue, oc_soft_dynamic,
     {create_sue, act_creature, draw_motion, NULL,
      set_action_sue, set_motion_sue, NULL}},
// Ƥоݤξ夫Τ
// flamedrawpost_drawȤϿ
	{ot_flame, oc_soft_dynamic,
     {create_flame, act_flame, NULL, draw_motion,
      NULL, NULL, NULL}},



    {ot_lift, oc_hard_dynamic,
     {create_lift, act_lift, draw_lift, NULL, NULL, NULL, NULL}},
    {ot_mswitch, oc_hard_dynamic,
     {create_mswitch, act_mswitch, draw_switch, NULL,
      NULL, NULL, NULL}},
    {ot_aswitch, oc_hard_dynamic,
     {create_aswitch, act_aswitch, draw_switch, NULL,
      NULL, NULL, NULL}},




    //    {ot_mvfloor, oc_hard_dynamic,
    //     {create_mvfloor, move_mvfloor, draw_mvfloor}},
    //	{ot_block, oc_static,
    //     {NULL, NULL, draw_block}},
    //    {ot_sting, oc_static,
    //     {NULL, NULL, draw_sting}},
};
obj_funcs_t obj_funcs_table[ot_end];
obj_class_t obj_class_table[ot_end];
#define TypeToClass(type) (obj_class_table[type])

creature_images_t creature_image_table[ot_chee+1];
creature_image_info_t creature_image_info_list[] = {
	{ot_hatopo,
     {
      ii_hatopo_shl1, ii_hatopo_shr1, 
      ii_hatopo_shl2, ii_hatopo_shr2, 
      ii_hatopo_svl1, ii_hatopo_svr1, 
      ii_hatopo_svl2, ii_hatopo_svr2, 
      ii_hatopo_pl, ii_hatopo_pr,
	  ii_hatopo_bnl, ii_hatopo_bnr,
      ii_bang8_8
	}},
    {ot_gagga,
     {
	  ii_none, ii_none,   // !!!
	  ii_none, ii_none,
	  ii_none, ii_none,
	  ii_none, ii_none,
	  ii_gagga_p, ii_gagga_p,
      ii_gagga_bn, ii_gagga_bn,
      ii_bang8_8
	}},
	{ot_yaarashi,
     {
	  ii_none, ii_none,   // !!!
	  ii_none, ii_none,
	  ii_none, ii_none,
	  ii_none, ii_none,
	  ii_yaarashi_pl, ii_yaarashi_pr,
      ii_yaarashi_bnl, ii_yaarashi_bnr,
      ii_bang8_4
	}},
    // !!! Sue, OjigiriγƲ
	{ot_sue,
     {
	  ii_none, ii_none,    // !!!
	  ii_none, ii_none,
	  ii_none, ii_none,
	  ii_none, ii_none,
	  ii_sue_pl, ii_sue_pr,
      ii_sue_pl, ii_sue_pr,  // !!!
      ii_bang12_8
	}},
	{ot_ojigiri,
     {
	  ii_ojigiri_shl1, ii_ojigiri_shr1, 
      ii_ojigiri_shl2, ii_ojigiri_shr2, 
      ii_ojigiri_svl1, ii_ojigiri_svr1, 
      ii_ojigiri_svl2, ii_ojigiri_svr2, 
	  ii_ojigiri_l, ii_ojigiri_r,
      ii_ojigiri_bnl, ii_ojigiri_bnr,
      ii_bang8_16
	}},
	{ot_gondin,
     {
	  ii_gondin_sh1, ii_gondin_sh1, 
      ii_gondin_sh2, ii_gondin_sh2, 
      ii_gondin_sv1, ii_gondin_sv1, 
      ii_gondin_sv2, ii_gondin_sv2,
	  ii_none, ii_none, // ˤϻɤʤ
	  ii_gondin_bn, ii_gondin_bn,
      ii_bang8_8
	}},
};
#define CREATURE_IMAGE_INFO_NUM \
    (sizeof(creature_image_info_list)/sizeof(creature_image_info_t))

// ͭset_motionؿset_motion_creatureǤĤcase
#define SkipDefaultCreatureMotions \
    case os_smash_h1: \
    case os_smash_h2: \
    case os_smash_v1: \
    case os_smash_v2: \
    case os_pierced: \
    case os_burned: \
    case os_bang: \
        break;


static obj_t *chee = NULL;
static obj_t *goal = NULL;



void obj_init(void){
    int i;

    for (i = 0 ; i < sizeof(obj_info)/sizeof(obj_info_t) ; i++){
        assert(obj_info[i].type >= ot_begin);
        assert(obj_info[i].type < ot_end);
        obj_class_table[obj_info[i].type] = obj_info[i].oclass;
        obj_funcs_table[obj_info[i].type] = obj_info[i].funcs;
    }
    for (i = 0 ; i < CREATURE_IMAGE_INFO_NUM ; i++){
        creature_image_table[creature_image_info_list[i].type] 
            = creature_image_info_list[i].images;
    }
    setup_obj_list();
}

void setup_obj_list(void){
    int i;

    clear_obj_req_queue();
	// ٤°ʤɤĤʤ褦˥֥ΰꥢƤ
    memset(objs, 0, sizeof(objs));
		for (i = oc_begin ; i < oc_end ; i++){
        obj_alive_list[i] = NULL;
    }
    obj_free_list = objs;
    for (i = 0 ; i < OBJ_NUM_MAX-1 ; i++){
        objs[i].next = &objs[i+1];
        objs[i+1].prev = &objs[i];
    }
    objs[0].prev = objs[OBJ_NUM_MAX-1].next = NULL;
}

obj_t *pop_obj_list(obj_t **pop_list){
    obj_t *obj;

    assert(*pop_list != NULL);
    
    obj = *pop_list;
    *pop_list = (*pop_list)->next;
    (*pop_list)->prev = NULL;
    return obj;
}
void push_obj_list(obj_t *obj, obj_t **push_list){
    obj->prev = NULL;
    obj->next = *push_list;

    if (*push_list != NULL){
        (*push_list)->prev = obj;
    }
    *push_list = obj;
}


obj_t *create_obj_grid(obj_type_t type, int x, int y, dir_t dir){
    return create_obj(type, x*GRID_SIZE, y*GRID_SIZE, dir);
}

obj_t *create_obj(obj_type_t type, int x, int y, dir_t dir){
    obj_t *new_obj;
    obj_class_t class;

    assert(obj_free_list != NULL);

    class = TypeToClass(type);

    //    printf("%d, %d\n", x, y);
    new_obj = pop_obj_list(&obj_free_list);
    new_obj->class = class;
    new_obj->type = type;
    new_obj->dir = dir;
    new_obj->x = x;
    new_obj->y = y;
    new_obj->width = GRID_SIZE;
    new_obj->height = GRID_SIZE;
	new_obj->link = NULL;
    new_obj->attr = 0;
	// uեɤͤĤʤ褦ˤ٤0ꥢƤ
	memset(&new_obj->u, 0, sizeof(new_obj->u));
	// !!! 1ե졼ܤpush_horizontalʤΤɤλ
	new_obj->u.creature.is_landed = 1;
    SetState(new_obj, os_none);
	if (type != ot_chee){
	    push_obj_list(new_obj, &obj_alive_list[class]);
	}
    if (obj_funcs_table[new_obj->type].create){
        obj_funcs_table[new_obj->type].create(new_obj, x, y, dir);
    }

    return new_obj;
}

obj_t *create_sign_grid(obj_type_t type, int x, int y, int link_no){
    obj_t *new_obj;

	new_obj = create_obj_grid(type, x, y, di_none);
	new_obj->u.sign.no = link_no;
	return new_obj;
}


void delete_obj(obj_t *obj){
    assert(obj != NULL);
    //    printf("delete obj obj:%p\n", obj);
    //    printf("delete obj prev:%p\n", 
    //           obj->prev);
    //    printf("delete obj next:%p\n", 
    //           obj->next);
    //    printf("delete obj obj:%p oc:%d prev:%p next:%p\n", 
    //           obj, obj->class, obj->prev, obj->next);
    //    printf("delete obj obj:%p oc:%d prev:%p next:%p\n", 
    //           obj, obj->class, obj->prev, obj->next);
    if (obj->prev == NULL){
        obj_alive_list[obj->class] = obj->next;
    }
    else {
        obj->prev->next = obj->next;
    }
    if (obj->next != NULL){
        obj->next->prev = obj->prev;
    }
    
    push_obj_list(obj, &obj_free_list);
}

void forall_obj(int(*f)(obj_t *), obj_class_t class){
    obj_t *obj = obj_alive_list[class];
    obj_t *next;
    int r;
    while (obj){
        r = f(obj);
        if (r){ // f0֤ȤϡΥ֥Ȥ
            next = obj->next;
            delete_obj(obj);
            obj = next;
        }
        else {
            obj = obj->next;
        }
    }
}

/*
void act_obj_list(void){
    forall_obj(act_obj, oc_hard_dynamic);
    forall_obj(act_obj, oc_soft_dynamic);
}
*/
int act_obj(obj_t *obj){
    if (obj_funcs_table[obj->type].act){
        return obj_funcs_table[obj->type].act(obj);
    }
    // actؿƤФʤʤ顢֥ȺⵯʤΤ0֤
    return 0;
}

void draw_obj_list(void){
    forall_obj(draw_obj, oc_static);
    forall_obj(draw_obj, oc_hard_dynamic);
    forall_obj(draw_obj, oc_soft_dynamic);
	draw_chee(chee);
    forall_obj(post_draw_obj, oc_hard_dynamic);
    forall_obj(post_draw_obj, oc_soft_dynamic);
}
int draw_obj(obj_t *obj){
    if (obj_funcs_table[obj->type].draw){
		obj_funcs_table[obj->type].draw(obj);
	}
    return 0;
}
int post_draw_obj(obj_t *obj){
    if (obj_funcs_table[obj->type].post_draw){
		obj_funcs_table[obj->type].post_draw(obj);
	}
    return 0;
}

void clear_obj_req_queue(void){
    req_num = 0;
}

void push_obj_req(obj_t *dist, obj_t *src, obj_state_t state){
    assert(req_num < OBJ_STATE_REQ_MAX);
    req_queue[req_num].dist = dist;
    req_queue[req_num].src = src;
    req_queue[req_num].state = state;
    req_num ++;
}
void apply_all_req(void){
    obj_t *src, *dist;
    obj_state_t state;
    while (req_num > 0){
        req_num --;
        dist = req_queue[req_num].dist;
        src = req_queue[req_num].src;
        state = req_queue[req_num].state;
        printf("apply req to state %d (%p->%p)\n", state, src, dist);
		// ῩоݤǤῩѤߤʤ饭󥻥
        if (state == os_eaten && 
            (State(dist) == os_eaten || State(dist) == os_digested))
        {
            SetState(src, os_cancel);
        }
		// Ǥ˥ɥƤʤٳʤ
        /*
        if (state == os_goal_opening){
			if (State(dist) == os_none){
				SetState(dist, state);
			}
        }
        else{
            SetState(dist, state);
        }
        */
        SetState(dist, state);
    }
}



void create_chee(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = CHEE_SIZE;
    obj->height = CHEE_SIZE;
	SetAttr(obj, at_meat);
    obj->u.creature.move_x = 0;
    obj->dir = dir;
    obj->motion = (dir == di_left) ? ii_chee_wl0 : ii_chee_wr0;
	chee = obj;
}


void set_chee(obj_t *obj){
	chee = obj;
}

obj_t *get_chee(void){
    return chee;
}

void create_gondin(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = GONDIN_SIZE;
    obj->height = GONDIN_SIZE;
	SetAttr(obj, at_vegetable);
	SetAttr(obj, at_immune_pierce);
	obj->motion = ii_gondin;
}

void create_hatopo(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = HATOPO_SIZE;
    obj->height = HATOPO_SIZE;
	SetAttr(obj, at_meat);
    obj->dir = dir;
    obj->u.creature.move_x = (dir == di_left) ? -1 : 1;
    obj->motion = (dir == di_left) ? ii_hatopo_wl0 : ii_hatopo_wr0;
	obj->u.creature.is_walking = 1; // ľȿžʤ褦ˤ뤿
}

void create_yaarashi(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = YAARASHI_WIDTH;
    obj->height = YAARASHI_HEIGHT;
	SetAttr(obj, at_meat);
    obj->dir = dir;
    obj->u.creature.move_x = (dir == di_left) ? -1 : 1;
    obj->motion = (dir == di_left) ? ii_yaarashi_wl0 : ii_yaarashi_wr0;
	SetAttr(obj, at_immune_pierce);
	obj->u.creature.is_walking = 1; // ľȿžʤ褦ˤ뤿
}

void create_gagga(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = GAGGA_SIZE;
    obj->height = GAGGA_SIZE;
	SetAttr(obj, at_meat);
    obj->motion = ii_gagga_n0;
	obj->dir = di_left;
}
void create_ojigiri(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = OJIGIRI_WIDTH;
    obj->height = OJIGIRI_HEIGHT;
	SetAttr(obj, at_vegetable);
	SetAttr(obj, at_rooted);
    obj->dir = dir;
    obj->motion = (dir == di_left) ? ii_ojigiri_l : ii_ojigiri_r;
}
void create_sue(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = SUE_WIDTH;
    obj->height = SUE_HEIGHT;
	SetAttr(obj, at_meat);
    obj->dir = dir;
    obj->motion = (dir == di_left) ? ii_sue_hcl : ii_sue_hcr;
	SetState(obj, os_head_high);
}
void create_flame(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = FLAME_WIDTH;
    obj->height = FLAME_HEIGHT;
    obj->dir = dir;
    obj->motion = (dir == di_left) ? ii_flame_ml : ii_flame_mr;
    set_emap_box(obj->x, obj->y, obj->width, obj->height, eb_flame);
}
void create_nya(obj_t *obj, int ox, int oy, dir_t dir){
	int px, ex;
	obj->width = NYA_WIDTH;
    obj->height = NYA_HEIGHT;
    obj->dir = dir;
	if (dir == di_left){
		obj->motion = ii_nya_l;
		px = ox-4;
		ex = ox+obj->width-1-3;
	}
	else {
		obj->motion = ii_nya_r;
		px = ox+8;
		ex = ox;
	}
	obj->link = create_obj(ot_punch, px, oy+8+1, dir);
	obj->link->link = obj;
	// Τblock°Ĥ
    set_emap_static_blockbox(ox, oy+2, obj->width, obj->height-2);
	set_emap_static_blockbox(ex, oy, 3, 2);
}
void create_punch(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = PUNCH_WIDTH;
    obj->height = PUNCH_HEIGHT;
    obj->dir = dir;
	obj->motion = (dir == di_left) ? ii_punch_l : ii_punch_r;
}


void create_wall(obj_t *obj, int ox, int oy, dir_t dir){
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
}

void create_goal(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = GOAL_WIDTH;
    obj->height = GOAL_HEIGHT;
	obj->motion = ii_goal_closed;
	goal = obj;
}

void create_sign(obj_t *obj, int ox, int oy, dir_t dir){
	obj->width = SIGN_WIDTH;
	obj->height = SIGN_HEIGHT;
    set_emap_static_box(obj->x, obj->y, obj->width, obj->height, eb_sign);
	set_emap_owner_static_box(obj->x, obj->y, obj->width, obj->height, eb_sign, obj);
}

void create_sting(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    switch (dir){
    case di_left:
        obj->type = ot_sting_l;
        for (x = 0 ; x < obj->width ; x++){
            SetEMapStatic(ox+x, oy+obj->height/2-1, eb_pierce);         
            SetEMapStatic(ox+x, oy+obj->height/2, eb_pierce);         
            SetEMapStatic(ox+x, oy+obj->height/2-1, eb_block_right);
            SetEMapStatic(ox+x, oy+obj->height/2, eb_block_right);
            //            SetEMapStatic(ox+x, oy+obj->height/2-1, block_down);
            //            SetEMapStatic(ox+x, oy+obj->height/2, block_up);
        }
        set_emap_static_box(ox+obj->width, oy+1, 1, obj->height-2, 
                            eb_block_left);
        set_emap_static_box(ox+obj->width-1, oy+1, 1, obj->height-2,
                            eb_block_up);
        set_emap_static_box(ox+obj->width-1, oy+1, 1, obj->height-2,
                            eb_block_down);
        break;
    case di_right:
        obj->type = ot_sting_r;
        for (x = 0 ; x < obj->width ; x++){
            SetEMapStatic(ox+x, oy+obj->height/2-1, eb_pierce);         
            SetEMapStatic(ox+x, oy+obj->height/2, eb_pierce);         
            SetEMapStatic(ox+x, oy+obj->height/2-1, eb_block_left);         
            SetEMapStatic(ox+x, oy+obj->height/2, eb_block_left);         
            //            SetEMapStatic(ox+x, oy+obj->height/2-1, block_down);
            //            SetEMapStatic(ox+x, oy+obj->height/2, block_up);
        }
        set_emap_static_box(ox-1, oy+1, 1, obj->height-2, eb_block_right);
        set_emap_static_box(ox, oy+1, 1, obj->height-2, eb_block_up);
        set_emap_static_box(ox, oy+1, 1, obj->height-2, eb_block_down);
        break;
    case di_up:
        obj->type = ot_sting_u;
        for (y = 0 ; y < obj->height ; y++){
            SetEMapStatic(ox+obj->width/2-1, oy+y, eb_pierce);         
            SetEMapStatic(ox+obj->width/2, oy+y, eb_pierce);         
            SetEMapStatic(ox+obj->width/2-1, oy+y, eb_block_down);
            SetEMapStatic(ox+obj->width/2, oy+y, eb_block_down);         
            //            SetEMapStatic(ox+obj->width/2-1, oy+y, block_right);
            //            SetEMapStatic(ox+obj->width/2, oy+y, block_left);         
        }
        set_emap_static_box(ox+1, oy+obj->height, obj->width-2, 1,
                            eb_block_up);
        set_emap_static_box(ox+1, oy+obj->height-1, obj->width-2, 1,
                            eb_block_left);
        set_emap_static_box(ox+1, oy+obj->height-1, obj->width-2, 1,
                            eb_block_right);
        break;
    case di_down:
        obj->type = ot_sting_d;
        for (y = 0 ; y < obj->height ; y++){
            SetEMapStatic(ox+obj->width/2-1, oy+y, eb_pierce);         
            SetEMapStatic(ox+obj->width/2, oy+y, eb_pierce);         
            SetEMapStatic(ox+obj->width/2-1, oy+y, eb_block_up);         
            SetEMapStatic(ox+obj->width/2, oy+y, eb_block_up);         
            //            SetEMapStatic(ox+obj->width/2-1, oy+y, block_right);
            //            SetEMapStatic(ox+obj->width/2, oy+y, block_left);         
        }
        set_emap_static_box(ox+1, oy-1, obj->width-2, 1, eb_block_down);
        set_emap_static_box(ox+1, oy, obj->width-2, 1, eb_block_left);
        set_emap_static_box(ox+1, oy, obj->width-2, 1, eb_block_right);
        break;
    default:
        assert(0);
        break;
    }
    obj->dir = dir;
}
void create_fwall(obj_t *obj, int ox, int oy, dir_t dir){
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
	obj->motion = ii_fwall;
}
void create_iwall(obj_t *obj, int ox, int oy, dir_t dir){
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
	obj->motion = ii_iwall;
}
void create_barrier(obj_t *obj, int ox, int oy, dir_t dir){
    SetState(obj, os_on); // !!!
	obj->motion = ii_barrier0;
    set_emap_blockbox(ox, oy, obj->width, obj->height);
}
void create_slope_l(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    for (y = 0 ; y < obj->height ; y++){
        for (x = obj->width-y-1 ; x < obj->width ; x++){
            SetEMapStatic(ox+x, oy+y-1, eb_block_down); 
            SetEMapStatic(ox+x, oy+y, eb_block_right); 
            SetEMapStatic(ox+x, oy+y-1, eb_push_leftdown);
        }
    }
}
void create_slope_r(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    for (y = 0 ; y < obj->height ; y++){
//        for (x = obj->width-y-1 ; x < obj->width ; x++){
        for (x = 0 ; x < y+1 ; x++){
            SetEMapStatic(ox+x, oy+y-1, eb_block_down); 
            SetEMapStatic(ox+x, oy+y, eb_block_left); 
            SetEMapStatic(ox+x, oy+y-1, eb_push_rightdown);
        }
    }
/*
int i;
    for (i = 0 ; i < obj->width ; i++){
        set_emap_static_blockbox(ox, oy+i,
                                 i+1, obj->height-i);
        SetEMapStatic(ox+i, oy+i-1, eb_push_rightdown);
    }
	*/
}
void create_stairs_l(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    for (y = 0 ; y < obj->height ; y++){
        for (x = obj->width-y-1 ; x < obj->height ; x++){
            SetEMapStatic(ox+x, oy+y-1, eb_block_down); 
            SetEMapStatic(ox+x-1, oy+y, eb_block_right); 
        }
        //        set_emap_static_blockbox(ox+obj->width-i-1, oy+i,
        //                                 obj->width-i-1, obj->height-i);
    }
}
void create_stairs_r(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    for (y = 0 ; y < obj->height ; y++){
        for (x = 0 ; x <= y ; x++){
            SetEMapStatic(ox+x, oy+y-1, eb_block_down); 
            SetEMapStatic(ox+x+1, oy+y, eb_block_left); 
        }
        //        set_emap_static_blockbox(ox, oy+i,
        //                                 i+1, obj->height-i);
    }
}
void create_lift(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = LIFT_DEFAULT_WIDTH;
    obj->dir = dir;
	SetState(obj, os_on);
    set_emap_blockbox(ox, oy, obj->width, obj->height);
}

void create_mswitch(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = SWITCH_DEFAULT_WIDTH;
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
	SetState(obj, os_off);
    obj->motion = ii_mswitch_off;
}

void create_aswitch(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = SWITCH_DEFAULT_WIDTH;
    //    aswitchϹ⤵ѤΤǡstaticemapϤʤ
    //    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
    SetState(obj, os_nonweighted_off);
    obj->motion = ii_aswitch_off;
}

void set_grid_width_static(obj_t *obj, int width){
    obj->width = width * GRID_SIZE;
    set_emap_static_blockbox(obj->x, obj->y, obj->width, obj->height);
	// !!! emapνȺޤǡŪdynamic emapˤ񤤤Ƥ
    set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
}

void set_grid_width_dynamic(obj_t *obj, int width){
    obj->width = width * GRID_SIZE;
	// ȤꤢĹΤΤblock°Ȳ
	// liftactꤹ뤫餿֤פϤʤϤ
    set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
}
void set_grid_height_dynamic(obj_t *obj, int height){
    obj->height = height * GRID_SIZE;
	// ȤꤢĹΤΤblock°Ȳ
	// lift, barrieractꤹ뤫餿֤פϤʤϤ
    set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
}

// եȤʤɤˤǤϡ֤
// Ǥos_none, Ǥʤ(٤줿)Ȥos_smash_xx֤
obj_state_t exclude_obj(obj_t *obj){
	int left_moved = 0, right_moved = 0;

    while (check_emap_obj_any_right(obj, eb_exclude_left)){
        obj->x --;
		left_moved ++;
        //        puts("exclude left");
    }
    while (check_emap_obj_any_left(obj, eb_exclude_right)){
        obj->x ++;
		right_moved ++;
        //        puts("exclude right");
    }
	if (CheckAttr(obj, at_rooted)){ // ꥿פʤ麸˲ʤ
		// ư餽ʬ٤줿Ȥߤʤ
		// 麸ɤ˲դ줿⤷ʤ
		// ξⱦ鲡줿ʸϱưϤʤΤǡ
		// ٤줿̤right_movedȽǤǤ롣
		// ٤쥰եåκüexclude_right줿֤Ǥ褤
		if (right_moved > 1){
            return os_smash_h2;
		}
		else if (right_moved == 1){
            return os_smash_h1;
		}
		// ưʤä顢鲡ƤʤΤ߲줿
		// Ԥξ硢֤򸵤ᤷ٤̤ꤹ
		else {
			obj->x += left_moved;
			if (left_moved > 1){
                return os_smash_h2;
			}
			else if (left_moved == 1){
                return os_smash_h1;
			}
		}
	}
    if (check_emap_obj_any_right(obj, eb_exclude_left)){
        if (check_emap_box_any_right(obj->x-1, obj->y, obj->width, obj->height,
                                    eb_exclude_left))
        {
            return os_smash_h2;
        }
        return os_smash_h1;
    }
    while (check_emap_obj_any_down(obj, eb_exclude_up)){
        obj->y --;
        printf("exclude up %d\n", obj->y);
    }
    while (check_emap_obj_any_up(obj, eb_exclude_down)){
        obj->y ++;
        printf("exclude down %d\n", obj->y);
    }
    if (check_emap_obj_any_down(obj, eb_exclude_up)){
        printf("smash v1 or v2\n");
        if (check_emap_box_any_down(obj->x, obj->y-1, obj->width, obj->height,
                                    eb_exclude_up))
        {
            printf("smash v2\n");
            return os_smash_v2;
        }
        printf("smash v1\n");
        return os_smash_v1;
    }
    return 0;
}

// ԥסڶΥ֥Ȥ򺸱˰ư
// ˰ưǤ0Ǥʤ0֤
int move_obj_horizontal(obj_t *obj, int xoffset){
    if (xoffset < 0){
        // ¦˾㳲ʪʤаư
        if (!check_emap_dir(eb_block_left, di_left, obj)){
            obj->x --;
            return 1;
        }
    }
    else if (xoffset > 0){
        // ¦˾㳲ʪʤаư
        if (!check_emap_dir(eb_block_right, di_right, obj)){
            obj->x ++;
            return 1;
        }
    }
    return 0;
}

// ԥפΥ֥Ȥ򺸱⤫
// ϤƤ뤳Ȥ󡣳ʤϼưǾ롣
// ⤤­줬ʤȤäƤΤǡ
// ԾְʳǻȤäƤϤʤʤ
// ⤱0⤱ʤ0֤
int walk_obj(obj_t *obj, int xoffset){
    if (xoffset < 0){
        // ¦˾㳲ʪʤаư
        if (!check_emap_dir(eb_block_left, di_left, obj)){
            obj->x --;
            // ⤤­줬ʤСΥ1ñ
            if (!check_emap_dir(eb_block_down, di_down, obj)){
                obj->y ++;
            }
            return 1;
        }
        // 1ʤΤ߾㳲ʪǾ˾㳲ʪʤСʤȸʤƾ
		// ̤ʤ㳲ʪˤ
        else if (CheckEMap(Lbx(obj), Lby(obj), eb_block_left) &&
                 !CheckEMap(Lbx(obj), Lby(obj)-1, eb_block_left) &&
                 !CheckEMap(Lbx(obj), Lby(obj)-1, eb_push_rightdown) &&
                 !check_emap_dir(eb_block_up, di_up, obj))
        {
            obj->x --;
            obj->y --;
            return 1;
        }
    }
    else if (xoffset > 0){
        // ¦˾㳲ʪʤаư
        if (!check_emap_dir(eb_block_right, di_right, obj)){
            obj->x ++;
            // ⤤­줬ʤСΥ1ñ
            if (!check_emap_dir(eb_block_down, di_down, obj)){
                obj->y ++;
            }
            return 1;
        }
        // 1ʤΤ߾㳲ʪǾ˾㳲ʪʤСʤȸʤƾ
		// ̤ʤ㳲ʪˤ
        else if (CheckEMap(Rbx(obj), Rby(obj), eb_block_right) &&
                 !CheckEMap(Rbx(obj), Rby(obj)-1, eb_block_right) &&
				 !CheckEMap(Rbx(obj), Rby(obj)-1, eb_push_leftdown) &&
                 !check_emap_dir(eb_block_up, di_up, obj))
        {
            obj->x ++;
            obj->y --;
            return 1;
        }
    }
    return 0;
}

// ζư
// !!! ȤꤢեѤʤΤǡưȤ­(down)åʤ
//     ʤɤʤ顢4å褦˽
// ư̤֤
int push_horizontal(obj_t *obj){
	int x, move_x=0;
    if (IsLanded(obj)){
		for (x = Lbx(obj) ; x <= Rbx(obj) ; x++){
			// ­줬
			if (CheckEMap(x, Lby(obj), eb_block_down)){
				// եȾʤȤꤢư
				// push_left/rightŤʤꤷ
				// ٤ƲûƤ
				// 껦ʤ鶯ưʤǤ褤
				if (CheckEMap(x, Lby(obj), eb_push_left)){
					move_x --;
				}
				else if (CheckEMap(x, Lby(obj), eb_push_right)){
					move_x ++;
				}
				// եȤǤʤ­ˤäƤʤ
				// (⤷ꤵƤʤ)ư
				// åϽλ
				else {
					move_x = 0;
					break;
				}
			}
		}
	}
	if (move_x == 0){
        obj->u.creature.inertia_x = 0;
	    return 0;
	}
	else if (move_x < 0){
		if (!check_emap_dir(eb_block_left, di_left, obj)){
            //	        printf ("push left\n");
		    obj->x --;
            //			// ξΤ(줫饸פݤ)
            //			if (IsLanded(obj)){
                obj->u.creature.inertia_x = -1;
                //                printf("inertia_x=%d\n", obj->u.creature.inertia_x);
                //            }
			return -1;
		}
	}
	else {
		if (!check_emap_dir(eb_block_right, di_right, obj)){
            //	        printf ("push right\n");
		    obj->x ++;
            //			// ξΤ(줫饸פݤ)
            //			if (IsLanded(obj)){
                obj->u.creature.inertia_x = 1;
                //                printf("inertia_x=%d\n", obj->u.creature.inertia_x);
                //            }
	        return 1;
		}
	}
	return 0;
}


// ̤ζư
// ư̤֤
int push_diagonal(obj_t *obj){
    if (CheckEMap(Rbx(obj), Rby(obj), eb_push_leftdown)){
        obj->x--;
        obj->y++;
        obj->u.creature.is_sliding = 1;
        obj->u.creature.is_landed = 1;
        return 1;
    }
    else if (CheckEMap(Lbx(obj), Lby(obj), eb_push_rightdown)){
//        puts("slide right");
        obj->x++;
        obj->y++;
        obj->u.creature.is_sliding = 1;
        obj->u.creature.is_landed = 1;
        return 1;
    }
    obj->u.creature.is_sliding = 0;
    return 0;
}

// äˤФ
// ФϽȡФζư
// Фʤ1֤
int blast_horizontal(obj_t *obj){
//   	if (check_emap_dir(eb_blast_left, di_right, obj)){
//	if (check_emap_obj_any_corner(obj, eb_blast_left)){
// ƱؤοФʤꤷʤ
// Ʊeb_blast2ե졼ʾƧǽꡢblast_countʣΤɤᡣ
// ñblast_x!=0ǤϡФ˵ؤοФ줬ȯʤʤ
// ФƱؤοФ줬Ƥ̵뤵뤬ɤΤߤ̤ʤʤ
	if (check_emap_obj_any_rim(obj, eb_blast_left) && (obj->u.creature.blast_x != -2)){
		puts("blast left");
		obj->u.creature.blast_x = -2;
		obj->u.creature.is_stunned = 1;
		if (obj->type == ot_chee){
			IncGameRecord(re_blast_count);
		}
	}
//    else if (check_emap_dir(eb_blast_right, di_left, obj)){
//	else if (check_emap_obj_any_corner(obj, eb_blast_right)){
	else if (check_emap_obj_any_rim(obj, eb_blast_right) & (obj->u.creature.blast_x != 2)){
		puts("blast right");
		obj->u.creature.blast_x = 2;
		obj->u.creature.is_stunned = 1;
		if (obj->type == ot_chee){
			IncGameRecord(re_blast_count);
		}
	}
	if (obj->u.creature.blast_x < 0){
		puts("blasting left");
		if (!check_emap_dir(eb_block_left, di_left, obj)){
			obj->x --;
		}
		else {
			puts("blasting left end");
			obj->u.creature.blast_x = 0;
			obj->u.creature.is_stunned = 0;
			set_emap_box(Ltx(obj)-4, Lty(obj), 4, obj->height, eb_shock);
		}
		if (!check_emap_dir(eb_block_left, di_left, obj)){
			obj->x --;
		}
		else {
			puts("blasting left end");
			obj->u.creature.blast_x = 0;
			obj->u.creature.is_stunned = 0;
			set_emap_box(Ltx(obj)-4, Lty(obj), 4, obj->height, eb_shock);
		}
		return 1;
	}
	else if (obj->u.creature.blast_x > 0){
		puts("blasting right");
		if (!check_emap_dir(eb_block_right, di_right, obj)){
			obj->x ++;
		}
		else {
			puts("blasting right end");
			obj->u.creature.blast_x = 0;
			obj->u.creature.is_stunned = 0;
			set_emap_box(Rtx(obj)+1, Rty(obj), 4, obj->height, eb_shock);
		}
		if (!check_emap_dir(eb_block_right, di_right, obj)){
			obj->x ++;
		}
		else {
			puts("blasting right end");
			obj->u.creature.blast_x = 0;
			obj->u.creature.is_stunned = 0;
			set_emap_box(Rtx(obj)+1, Rty(obj), 4, obj->height, eb_shock);
		}
		return 1;
	}
	return 0;
}


void set_motion_creature(obj_t *obj){
	// ޤŪơ֥Υ⡼
	switch (State(obj)){
	case os_smash_h1:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].smash_h1_l :
            creature_image_table[obj->type].smash_h1_r;  
		break;
	case os_smash_h2:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].smash_h2_l :
            creature_image_table[obj->type].smash_h2_r;  
		break;
	case os_smash_v1:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].smash_v1_l :
            creature_image_table[obj->type].smash_v1_r;  
		break;
	case os_smash_v2:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].smash_v2_l :
            creature_image_table[obj->type].smash_v2_r;  
		break;
	case os_pierced:
		printf("piereced motion\n");
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].pierced_l :
            creature_image_table[obj->type].pierced_r;  
		break;
	case os_burned:
		printf("burned motion\n");
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].burned_l :
            creature_image_table[obj->type].burned_r;  
		break;
	case os_bang:
		printf("bang motion\n");
		obj->motion = creature_image_table[obj->type].bang;
		break;
	default:
		break;
	}

	// ͭؿФǾ񤭤
	if (obj_funcs_table[obj->type].set_motion){
        obj_funcs_table[obj->type].set_motion(obj);
    }
}

int act_creature(obj_t *obj){
    obj_state_t sm;

    switch (State(obj)){
    case os_smash_h1:
    case os_smash_h2:
    case os_smash_v1:
    case os_smash_v2:
        printf("smash:%d\n", State(obj));
        SetState(obj, os_bang);
        break;
    case os_eaten:
        SetState(obj, os_digested);
        break;
    case os_digested:
        IncCtr(obj);
        if (Ctr(obj) == OJIGIRI_DIGEST_COUNT){
            puts("dead digested");
			inc_kill_record(obj->type);
            return 1;
        }
        break;
	case os_pierced:
	case os_burned:
		IncCtr(obj);
		if (Ctr(obj) == CREATURE_DEATH_COUNT){
			SetState(obj, os_bang);
		}
		break;
    case os_bang:
        puts("dead bang");
		inc_kill_record(obj->type);
        return 1;
    default:
        // ˿줿˴
        // Хꥢexclude뤫˽Ťʤ˴ȽꤷƤ
        if (check_emap_any_rim(eb_flame, obj)){
            SetState(obj, os_burned);        
            printf("burned!!!!\n");
            break;
        }
        sm = exclude_obj(obj);
        if (sm){
            SetState(obj, sm);
            // ٤줿ϰʲλ˴ưϥåפ⡼
            break;
        }
		// ˿줿˴
        if (!CheckAttr(obj, at_immune_pierce) &&
			check_emap_any_rim(eb_pierce, obj))
		{
            SetState(obj, os_pierced);
	        printf("pierced!!!!\n");
			break;
        }

        // եȤǺư
        push_horizontal(obj);

        if (obj_funcs_table[obj->type].set_action){
            if (obj_funcs_table[obj->type].set_action(obj)) return 1;
        }
		// ä˿Ф
		if (blast_horizontal(obj)){
		    // Ǥϡblast_horizontalǽƤޤΤǡ϶
		}
        // ̤
        else if (push_diagonal(obj)){
            // Ǥϡpush_diagonalǽƤޤΤǡ϶
        }
        // к⤯
        else if (check_emap_dir(eb_block_down, di_down, obj)){
            obj->u.creature.is_landed = 1;
			// set_actionǾ֤Ѳ뤳ȤΤǡưǽåɬ
            if (IsAlive(obj) && !obj->u.creature.is_stunned){
                obj->u.creature.is_walking
                    = walk_obj(obj, obj->u.creature.move_x);
            }
        }
        // ʤпľ
        else {
            obj->y ++;
            obj->u.creature.is_landed = 0;
            //        falling = 1;
        }
    }

	set_motion_creature(obj);
    if (obj_funcs_table[obj->type].set_emap){
        obj_funcs_table[obj->type].set_emap(obj);
    }
    if (IsAlive(obj)){
		if (CheckAttr(obj, at_meat)){
	        set_emap_box(obj->x, obj->y, obj->width, obj->height, eb_meat);
		    set_emap_owner_box(obj->x, obj->y, obj->width, obj->height,
			                   eb_meat, obj);
		}
		else if (CheckAttr(obj, at_vegetable)){
	        set_emap_box(obj->x, obj->y, obj->width, obj->height, eb_vegetable);
// !! vegetableФReqStateʤ
//		    set_emap_owner_box(obj->x, obj->y, obj->width, obj->height,
//			                   eb_vegetable, obj);
		}
	}
	set_emap_box(Lbx(obj), Lby(obj), obj->width, 1, eb_weight);
	set_emap_owner_box(Lbx(obj), Lby(obj), obj->width, 1, eb_weight, obj);
//    SetEMapBox(obj->x, obj->y, obj->width, obj->height, eat);
    return 0;
}

int act_chee(obj_t *obj){
    //    int move_x;
    obj_state_t st;
	int v;
	obj_t *target;
//	printf("is_landed:%d\n", IsLanded(obj));

    // ˴ν
    if (!IsAlive(obj)){
        //        printf("dead state %d dir:%d ctr:%d\n", State(obj), obj->dir, Ctr(obj));
        switch (State(obj)){
        case os_smash_h1:
        case os_smash_h2:
        case os_smash_v1:
        case os_smash_v2:
			CountRecord(re_killed_by_smash);
            SetState(obj, os_bang);
            obj->motion = ii_bang8_8;
            break;
        case os_eaten:
			if (Ctr(obj) == 0){
				CountRecord(re_killed_by_ojigiri);
			}
            IncCtr(obj);
            if (Ctr(obj) == OJIGIRI_SWALLOW_COUNT){
                SetState(obj, os_digested);
				obj->motion = ii_none; // ٤Ƥ֤ɽ
            }
            break;
        case os_digested:
            IncCtr(obj);
            if (Ctr(obj) == OJIGIRI_DIGEST_COUNT + OJIGIRI_VOMIT_COUNT){
                SetState(obj, os_ghost);
                if (obj->dir == di_left){
                    obj->motion = ii_chee_gl0;
                }
                else {
                    obj->motion = ii_chee_gr0;
                }
            }
            break;
        case os_exit:
			IncCtr(obj);
//            obj->motion = ii_chee_back;
            if (Ctr(obj) == CHEE_EXIT_COUNT){
                SetState(obj, os_clear);
				obj->motion = ii_none;
            }
			break;
		case os_clear:
			if (Ctr(obj) == 0) finalize_objective_stat();
			IncCtr(obj);
            if (Ctr(obj) == CHEE_CLEAR_COUNT){
                SetState(obj, os_show_result);
            }
            break;
        case os_pierced:
			if (Ctr(obj) == 0){
				CountRecord(re_killed_by_pierced);
			}
            IncCtr(obj);
            if (Ctr(obj) == CHEE_PIERCE_COUNT){
                SetState(obj, os_ghost);
                if (obj->dir == di_left){
                    obj->motion = ii_chee_gl0;
                }
                else {
                    obj->motion = ii_chee_gr0;
                }
            }
            break;
        case os_burned:
			if (Ctr(obj) == 0){
				CountRecord(re_killed_by_flame);
			}
            IncCtr(obj);
            if (Ctr(obj) == CHEE_BURN_COUNT){
                SetState(obj, os_ghost);
                if (obj->dir == di_left){
                    obj->motion = ii_chee_gl0;
                }
                else {
                    obj->motion = ii_chee_gr0;
                }
            }
            break;
        case os_bang:
            if (obj->dir == di_left){
                obj->motion = ii_chee_gl0;
            }
            else {
                obj->motion = ii_chee_gr0;
            }
            SetState(obj, os_ghost);
            break;
        case os_ghost:
            obj->y --;
            IncCtr(obj);
            if (Rby(obj) == 0 || Ctr(obj) == GHOST_COUNT){
                SetState(obj, os_gameover);
            }
            break;
		case os_show_result:
		case os_gameover:
            IncCtr(obj);
			obj->motion = ii_none; // ɽ
			break;
        default:
            printf("Error: Unknown CheeChee state %d\n", State(obj));
            break;
        }
        return 0;
    }

    // ˿줿˴
    if (check_emap_any_rim(eb_flame, obj)){
        SetState(obj, os_burned);        
        obj->motion = (obj->dir == di_left) ? ii_chee_bnl : ii_chee_bnr;  
        printf("burned!!!!\n");
        return 0;
    }
    st = exclude_obj(obj);
    if (st){
        switch (st){
        case os_smash_v1:
            obj->motion = (obj->dir == di_left) ? ii_chee_svl1 : ii_chee_svr1;
            break;
        case os_smash_v2:
            obj->motion = (obj->dir == di_left) ? ii_chee_svl2 : ii_chee_svr2;
            break;
        case os_smash_h1:
            obj->motion = (obj->dir == di_left) ? ii_chee_shl1 : ii_chee_shr1;
            break;
        case os_smash_h2:
            obj->motion = (obj->dir == di_left) ? ii_chee_shl2 : ii_chee_shr2;
            break;
        default:
            break;
        }
        SetState(obj, st);
        return 0;
    }

    if (State(obj) == os_eating){
        obj->motion = ii_chee_e0;
        IncCtr(obj);
        printf("chee eating:%d\n", Ctr(obj));
        if (Ctr(obj) == CHEE_EAT_COUNT){
            SetState(obj, os_none);
            obj->u.creature.is_stunned = 0;
            obj->motion = ii_chee_init;
			if (inc_gondin_eaten()){
				ReqState(goal, obj, os_goal_req_open);
			}
        }
    }
    if ((v = check_emap_pos_obj_any_corner(obj, eb_sign)))
    {
        target = GetEMapOwner(CheckVal2PosX(v), CheckVal2PosY(v), eb_sign);
        assert(target != NULL);
//		printf("found sign link=%d\n", target->u.sign.no);
		set_current_stage_sign_no(target->u.sign.no);
	}
	else {
		set_current_stage_sign_no(INVALID_LINK_NO);
	}

	if (!IsStunned(obj) &&
        ((v = check_emap_obj_any_rim(obj, eb_gondin)) ||
         (v = check_emap_box_any(obj->x+3, obj->y+3, 1, 1, eb_gondin))))
    {
        printf("v=%d\n", v);
        target = GetEMapOwner(CheckVal2PosX(v), CheckVal2PosY(v), eb_gondin);
        assert(target != NULL);
        ReqState(target, obj, os_gondin_eaten);
        puts("Gondin!!!");
        SetState(obj, os_eating);
        // ٤Ƥ֤ϽϤŨι뤬ưǽʤΤ
        // stunnedˤƤ
        obj->u.creature.is_stunned = 1;
        obj->motion = ii_chee_e0;
        return 0;
    }

    if (check_emap_obj_all_corner(obj, eb_goal) && !IsStunned(obj)){
        puts("Goal!!!");
        SetState(obj, os_exit);
        ReqState(goal, obj, os_exit);
        obj->motion = ii_chee_back;
        return 0;
    }
    // ˿줿˴
    if (check_emap_any_rim(eb_pierce, obj)){
        SetState(obj, os_pierced);        
        obj->motion = (obj->dir == di_left) ? ii_chee_pl : ii_chee_pr;  
        printf("pierced!!!!\n");
        return 0;
    }
	if (MoveX(obj) && !IsStunned(obj)){
		obj->dir = (MoveX(obj) < 0) ? di_left : di_right;
	}

    // եȤǺư
    if (push_horizontal(obj)){
		IncGameRecord(re_lift_frame);
	}

	// ä˿Ф
	if (blast_horizontal(obj)){
	}

    // ̤
    else if (push_diagonal(obj)){
        // Ǥϡpush_diagonalǽƤޤΤǡ϶
    }
    // 
    else if (obj->u.creature.float_time){
        //        printf("jumping jx=%d jy=%d\n", obj->u.creature.jump_x, obj->u.creature.jump_y);
        // 徺
//        if (obj->u.creature.jump_time){        
		if (obj->y > obj->u.creature.jump_y_limit){        
            if (!check_emap_dir(eb_block_up, di_up, obj)){
                obj->y --;
                // եȤδʬû
                // ΤȤϡͤȴʤ褦1ñ̤ľȽ&ư
                if (obj->u.creature.jump_y < 0){
                    if (!check_emap_dir(eb_block_up, di_up, obj)){
                        obj->y += obj->u.creature.jump_y;
                    }
                }
                else {
                    obj->y += obj->u.creature.jump_y;
                }
            }
            else {
                // ŷƬ֤ĤƤϤʤ
                // ˤ褱Ƥ⤽ʾϾ徺Ǥʤ
                //            obj->u.creature.jump_time = 0;
                // ˤäΥɤˤȡ
                //徺ŷ򤱤кƾ徺Ǥ褦ˤʤ
            }
            obj->u.creature.float_time --;
        }
        // 徺λڶ
        else {
            obj->u.creature.jump_y = 0;
            obj->u.creature.float_time --;
        }
        move_obj_horizontal(obj, obj->u.creature.move_x);
        
        // դʤ餽ʬ˰ư
        // ưǤʤä(=֤Ĥä)鴷ʬ®٤0ˤ
        if (obj->u.creature.jump_x){
            if (move_obj_horizontal(obj, obj->u.creature.jump_x) == 0){
                obj->u.creature.jump_x = 0;
            }
        }
    }
    // Ǥʤư֥åƤʤϤȤߤʤ
    // ľǤ⡢жϤξǥ׻Ȥ
    else if (check_emap_dir(eb_block_down, di_down, obj)){
        //        printf("landed walk:%d jump_x:%d\n", obj->u.creature.move_x, obj->u.creature.jump_x);
        obj->u.creature.is_landed = 1;
        obj->u.creature.jump_x = 0;
//        obj->u.creature.jump_y = 0;
        if (!obj->u.creature.is_stunned){
            if (walk_obj(obj, obj->u.creature.move_x)){
				IncGameRecord(re_walk_frame);
			}
        }
    }
    else { // ڶʳǲư֥åƤʤȤ
//        		printf("falling %d %d\n", obj->u.creature.jump_time, obj->u.creature.float_time);
        obj->y ++;
        // 1ñ̡­줬landed
        if (check_emap_dir(eb_block_down, di_down, obj)){
            puts("landed");
            obj->u.creature.is_landed = 1;
            //			// ߥեȤꤷϤľϥפ˲
//			obj->u.creature.jump_y = 1;
	        if (!obj->u.creature.is_stunned){
		        walk_obj(obj, obj->u.creature.move_x);
			}
        }
        else {
            obj->u.creature.is_landed = 0;
            move_obj_horizontal(obj, obj->u.creature.move_x);
            // դʤ餽ʬ˰ư
			   // ưǤʤä(=֤Ĥä)鴷ʬ®٤0ˤ
            if (obj->u.creature.jump_x){
                if (move_obj_horizontal(obj, obj->u.creature.jump_x) == 0){
                    obj->u.creature.jump_x = 0;
                }
            }
        }
    }
    // ľեȾΤȤδ
    //  ʬʤɤαƶȿǤ뤿ᡢΰ֤Ƚꤹ
    if (check_emap_obj_any_down(chee, eb_push_down)){
        obj->u.creature.inertia_y = 1;
    }
    else if (check_emap_obj_any_down(chee, eb_push_up)){
        obj->u.creature.inertia_y = -1;
    }
    else {
        obj->u.creature.inertia_y = 0;
    }
    //    printf("inertia y:%d\n", obj->u.creature.inertia_y);
        
    // ˥᡼ѥ

    // ϥϤʤС饤ǥ󥰻Ǹ
    // ϤСФ뤿̾ԥ˥᡼
    if (obj->u.creature.is_sliding && (obj->u.creature.move_x == 0)){
        obj->motion = (obj->dir ==  di_left) ? ii_chee_jl : ii_chee_jr;
    }
    else if (obj->u.creature.blast_x){
        obj->motion = (obj->dir ==  di_left) ? ii_chee_bl : ii_chee_br;
    }
    else if (obj->u.creature.is_landed){
//		printf("motion landed %d\n", obj->u.creature.move_x);
		if (obj->u.creature.move_x < 0){
			if (obj->motion == ii_chee_wl0){
				obj->motion = ii_chee_wl1;
	        }
		    else {
			    obj->motion = ii_chee_wl0;
			}
		}
		else if (obj->u.creature.move_x > 0){
			if (obj->motion == ii_chee_wr0){
				obj->motion = ii_chee_wr1;
	        }
		    else {
			    obj->motion = ii_chee_wr0;
			}
		}
        else { // move_x == 0
			obj->motion = (obj->dir == di_left) ? ii_chee_wl0 : ii_chee_wr0;
        }
		
	}
    else { // not landed
        if (obj->dir == di_right){
            if (!jump_key_released && obj->motion == ii_chee_jr){
//            if (obj->motion == ii_chee_jr){
                obj->motion = ii_chee_fr;
            }
            else {
                obj->motion = ii_chee_jr;
            }
        }
        else if (obj->dir == di_left){
            if (!jump_key_released  && obj->motion == ii_chee_jl){
//            if (obj->motion == ii_chee_jl){
                obj->motion = ii_chee_fl;
            }
            else {
                obj->motion = ii_chee_jl;
            }
        }
    }
//    printf("chee y=%d %d\n", obj->y, obj->y+obj->height-1);
    set_emap_box(obj->x, obj->y, obj->width, obj->height, eb_meat);
    set_emap_owner_box(obj->x, obj->y, obj->width, obj->height, eb_meat, obj);
	if (IsLanded(obj) && !IsStunned(obj)){
        //		printf("set weight.\n");
		set_emap_box(Lbx(obj), Lby(obj), obj->width, 1, eb_weight);
		set_emap_owner_box(Lbx(obj), Lby(obj), obj->width, 1, eb_weight, obj);
	}
	//    SetEMapBox(obj->x, obj->y, obj->width, obj->height, eat);
    return 0;
}

int set_action_gondin(obj_t *obj){
    if (State(obj) == os_gondin_eaten) return 1;
	return 0;
}
void set_motion_gondin(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
	/*
	case os_smash_h1:
        obj->motion = ii_gondin_sh1;
        break;
    case os_smash_h2:
        obj->motion = ii_gondin_sh2;
        break;
    case os_smash_v1:
        obj->motion = ii_gondin_sv1;
        break;
    case os_smash_v2:
        obj->motion = ii_gondin_sv2;
        break;
    case os_bang:
        obj->motion = ii_bang8_8;
        break;
		*/
    default:
		obj->motion = ii_gondin;
        break;
    }
}
void set_emap_gondin(obj_t *obj){
	// äȿ褦vegetable°ĤƤ
	// ojigiri_immuneʤΤǤ˿٤뤳ȤϤʤ
//	set_emap_box(obj->x, obj->y, obj->width, obj->height, eb_meat);
//    set_emap_owner_box(obj->x, obj->y, obj->width, obj->height, eb_meat, obj);
    set_emap_box(obj->x+3, obj->y+3, 2, 2, eb_gondin);
    set_emap_owner_box(obj->x+3, obj->y+3, 2, 2, eb_gondin, obj);
}


int set_action_hatopo(obj_t *obj){
    if (obj->u.creature.is_landed){
        // ⤱餽Τޤ⤭³
        if (obj->u.creature.is_walking){
        }
        // äƤϤʤˤ⤷ʤ
        else if (obj->u.creature.is_sliding){
        }
        // Ϥ⤱ʤϤ֤ĤäƤΤȿž
        else {
            //            printf("turn %d\n", obj->dir);
            obj->dir = ReverseDir(obj->dir);
            obj->u.creature.move_x = (obj->dir == di_left) ? -1 : 1;
        }
    }
	return 0;
}
void set_motion_hatopo(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
	/*
	case os_smash_h1:
        obj->motion = (obj->dir == di_left) ? ii_hatopo_shl1 : ii_hatopo_shr1;
        break;
    case os_smash_h2:
        obj->motion = (obj->dir == di_left) ? ii_hatopo_shl2 : ii_hatopo_shr2;
        break;
    case os_smash_v1:
        //        printf("smash_v1 %d\n", obj->y);
        obj->motion = (obj->dir == di_left) ? ii_hatopo_svl1 : ii_hatopo_svr1;
        break;
    case os_smash_v2:
        obj->motion = (obj->dir == di_left) ? ii_hatopo_svl2 : ii_hatopo_svr2;
        break;
		*/
    case os_digested:
        obj->motion = ii_none;
        break;
    default:
        if (obj->u.creature.is_sliding){
            obj->motion = (obj->dir ==  di_left) ? ii_hatopo_sl : ii_hatopo_sr;
        }
		else if (IsStunned(obj)){
            obj->motion = (obj->dir ==  di_left) ? ii_hatopo_bl : ii_hatopo_br;
		}
        // ­򳫤ޤ޸
        else if (!obj->u.creature.is_landed){
            obj->motion = (obj->dir ==  di_left) ?
                ii_hatopo_wl0 : ii_hatopo_wr0;
        }
        // Ի
        else {
            if (obj->dir == di_left){
                obj->motion = (obj->motion == ii_hatopo_wl0) ?
                    ii_hatopo_wl1 : ii_hatopo_wl0;
            }
            else if (obj->dir == di_right){
                obj->motion = (obj->motion == ii_hatopo_wr0) ?
                    ii_hatopo_wr1 : ii_hatopo_wr0;
            }
        }
        break;
    }
}
int set_action_yaarashi(obj_t *obj){
	// ɤᡢat_immune_pierceˤƤơ
	// ʢ¦ΤȽԤ
	if (check_emap_down(eb_pierce, obj)){
        SetState(obj, os_pierced);
		return 0;
	}
    if (obj->u.creature.is_landed){
        // ⤱顢⤭³褦Ȥ
		if (obj->u.creature.is_walking){
			//  &&
//			(((obj->dir == di_left) && CheckEMap(Lbx(obj)-1, Lby(obj), eb_block_down)) ||
//			((obj->dir == di_right) && CheckEMap(Rbx(obj)+1, Rby(obj), eb_block_down))))
//			printf("yaarashi walk %d.\n", obj->u.creature.is_walking);
		}
        // äƤϤʤˤ⤷ʤ
        else if (obj->u.creature.is_sliding){
        }
        // Ϥ⤱ʤäϤ֤ĤäƤ뤫­줬ʤΤȿž
        else {
//            printf("turn %d\n", obj->dir);
            obj->dir = ReverseDir(obj->dir);
		}
		// ­줬ʤС⤫ʤ
		// 䤢餷ϼӹߤʤ
		if (obj->dir == di_left){
			if (CheckEMap(Lbx(obj)-1, Lby(obj), eb_block_down)){
				obj->u.creature.move_x = -1;
			}
			else {
				obj->u.creature.move_x = 0;
			}
		}
		else {
			if (CheckEMap(Rbx(obj)+1, Rby(obj), eb_block_down)){
				obj->u.creature.move_x = 1;
			}
			else {
				obj->u.creature.move_x = 0;
			}
		}
   }
//	obj->u.creature.move_x = 0; // !!!
	return 0;
}
void set_motion_yaarashi(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
    default:
        if (obj->dir == di_left){
            obj->motion = (obj->motion == ii_yaarashi_wl0) ?
                ii_yaarashi_wl1 : ii_yaarashi_wl0;
        }
        else if (obj->dir == di_right){
            obj->motion = (obj->motion == ii_yaarashi_wr0) ?
                ii_yaarashi_wr1 : ii_yaarashi_wr0;
        }
        break;
    }
}
void set_emap_yaarashi(obj_t *obj){
    set_emap_box(Ltx(obj), Lty(obj)-2, obj->width, 2, eb_pierce);
}

int set_action_gagga(obj_t *obj){
	switch (State(obj)){
	case os_blast:
		IncCtr(obj);
		if (Ctr(obj) == GAGGA_HIT_COUNT){
			SetState(obj, os_none);
		}
		break;
	default:
        if (check_emap_box_any(Ltx(obj)-6-1, Lty(obj)+1, 
                               6, obj->height-1, eb_meat) ||
			check_emap_box_any(Ltx(obj)-6-1, Lty(obj)+1, 
                               6, obj->height-1, eb_vegetable))
        {
			obj->dir = di_left;
			SetState(obj, os_blast);
		}
		else if (check_emap_box_any(Rtx(obj)+1+1, Rty(obj)+1,
                                    6, obj->height-1, eb_meat) ||
				 check_emap_box_any(Rtx(obj)+1+1, Rty(obj)+1,
                                    6, obj->height-1, eb_vegetable))
        {
            obj->dir = di_right;
			SetState(obj, os_blast);
        }
		break;
	}
	return 0;
}
void set_motion_gagga(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
    default:
		break;
	}
}

// creature¦eb_blastȽany_cornerǤäƤΤ
void set_emap_gagga(obj_t *obj){
	if (State(obj) == os_blast && Ctr(obj) >= GAGGA_BLAST_COUNT){
		if (obj->dir == di_left){
		    set_emap_box(Ltx(obj)-4-1, Lty(obj)+1, 4, obj->height-1, eb_blast_left);
		    set_emap_box(Ltx(obj)-4-1, Lty(obj)+1, 4, obj->height-1, eb_shock);
		}
		else {
		    set_emap_box(Rtx(obj)+1+1, Rty(obj)+1, 4, obj->height-1, eb_blast_right);
		    set_emap_box(Rtx(obj)+1+1, Rty(obj)+1, 4, obj->height-1, eb_shock);
		}
	}
}
	
int set_action_nya(obj_t *obj){
	return 0;
}
void set_motion_nya(obj_t *obj){
}

int set_action_ojigiri(obj_t *obj){
    int v;
	obj_t *target;

	IncCtr(obj);
	switch (State(obj)){
//	case os_cancel:
//        	SetState(obj, os_none);
//            break;
	case os_eating:
        if (Ctr(obj) == OJIGIRI_SWALLOW_COUNT){
			if (obj->dir == di_left){
		        v = check_emap_pos_box_any(Ltx(obj)-4, Rby(obj)-2, 2, 2,
		                                   eb_meat);
			}
			else {
				v = check_emap_pos_box_any(Rtx(obj)+3, Rby(obj)-2, 2, 2,
                                           eb_meat);
			}
			if (v){
				printf("eat target of %x:(%d, %d)\n",
				       v, CheckVal2PosX(v), CheckVal2PosY(v));
				target = GetEMapOwner(CheckVal2PosX(v), CheckVal2PosY(v), eb_meat);
				assert(target != NULL);
				ReqState(target, obj, os_eaten);
			}
			else { // 
	        	SetState(obj, os_none);
			}
        }
        else if (Ctr(obj) >= OJIGIRI_EAT_COUNT){
			SetState(obj, os_digesting);
		}
		break;
	case os_digesting:
		if (Ctr(obj) >= OJIGIRI_DIGEST_COUNT){
			SetState(obj, os_vomitting);
		}
		break;
	case os_vomitting:
		if (Ctr(obj) >= OJIGIRI_VOMIT_COUNT){
			SetState(obj, os_none);
		}
		break;
	default:
		if (Ctr(obj) >= OJIGIRI_SWING_COUNT){
			SetCtr(obj, 0);
			obj->dir = ReverseDir(obj->dir);
		}
        if ((v = check_emap_pos_box_any(Ltx(obj)-4, Rby(obj)-2, 2, 2,
                                          eb_meat)))
        {
            obj->dir = di_left;
		    SetState(obj, os_eating);
        }
        else if ((v = check_emap_pos_box_any(Rtx(obj)+3, Rby(obj)-2, 2, 2,
                                               eb_meat)))
        {
            obj->dir = di_right;
		    SetState(obj, os_eating);
        }
		/*
		if (v){
//            printf("get target of %x:(%d, %d)\n",
//                   v, CheckVal2PosX(v), CheckVal2PosY(v));
//            obj->link = GetEMapOwner(CheckVal2PosX(v), CheckVal2PosY(v), eb_meat);
//            assert(obj->link != NULL);
		    SetState(obj, os_eating);
		}
			*/
		break;
    }
	return 0;
}

void set_motion_ojigiri(obj_t *obj){
	switch (State(obj)){
	SkipDefaultCreatureMotions;
	case os_none:
	    obj->motion = (obj->dir ==  di_left) ? ii_ojigiri_l : ii_ojigiri_r;
		break;
/*
	case os_smash_h1:
        obj->motion = (obj->dir == di_left) ? ii_ojigiri_shl1 : ii_ojigiri_shr1;
        break;
    case os_smash_h2:
        obj->motion = (obj->dir == di_left) ? ii_ojigiri_shl2 : ii_ojigiri_shr2;
        break;
    case os_smash_v1:
        obj->motion = (obj->dir == di_left) ? ii_ojigiri_svl1 : ii_ojigiri_svr1;
        break;
    case os_smash_v2:
        obj->motion = (obj->dir == di_left) ? ii_ojigiri_svl2 : ii_ojigiri_svr2;
        break;
	case os_digesting:
		obj->motion = (Ctr(obj) % 4 < 2) ? ii_ojigiri_d0 : ii_ojigiri_d1;
		break;
	case os_bang:
		obj->motion = ii_bang8_16;
		break;
		*/
	default:
		break;
	}
}

int set_action_sue(obj_t *obj){
	IncCtr(obj);
	switch (State(obj)){
	case os_head_high:
		if (Ctr(obj) >= SUE_WAIT_COUNT) SetState(obj, os_breath_high);
		break;
	case os_head_low:
		if (Ctr(obj) >= SUE_WAIT_COUNT) SetState(obj, os_breath_low);
		break;
	case os_breath_high:
		if (obj->dir == di_left){
				create_obj(ot_flame, Ltx(obj)-FLAME_WIDTH, Lty(obj), di_left);
		}
		else {
				create_obj(ot_flame, Rtx(obj)+1, Lty(obj), di_right);
		}
		if (Ctr(obj) >= SUE_BREATH_COUNT) SetState(obj, os_head_low);
		break;
	case os_breath_low:
		if (obj->dir == di_left){
				create_obj(ot_flame, Ltx(obj)-FLAME_WIDTH, Lty(obj)+4, di_left);
		}
		else {
				create_obj(ot_flame, Rtx(obj)+1, Lty(obj)+4, di_right);
		}
		if (Ctr(obj) >= SUE_BREATH_COUNT) SetState(obj, os_head_high);
		break;
    default:
        break;
	}
	return 0;
}
	
void set_motion_sue(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
	case os_head_high:
		obj->motion = (obj->dir == di_left) ? ii_sue_hcl : ii_sue_hcr;
		break;
	case os_head_low:
		obj->motion = (obj->dir == di_left) ? ii_sue_lcl : ii_sue_lcr;
		break;
	case os_breath_high:
		obj->motion = (obj->dir == di_left) ? ii_sue_hol : ii_sue_hor;
		break;
	case os_breath_low:
		obj->motion = (obj->dir == di_left) ? ii_sue_lol : ii_sue_lor;
		break;
    default:
		break;
	}
}

int act_punch(obj_t *obj){
	int turn = 0;

	switch (obj->dir){
    case di_left:
        if (!check_emap_dir(eb_block_left, di_left, obj)){
            obj->x --;
        }         
        else {
            set_emap_box(Ltx(obj)-1, Lty(obj), 4, obj->height, eb_shock);
            obj->dir = di_right;
			turn = 1;
        }
        break;
    case di_right:
        if (!check_emap_dir(eb_block_right,di_right, obj)){
            obj->x ++;
        }         
        else {
            set_emap_box(Rbx(obj)+1, obj->y, 4, obj->height, eb_shock);
            obj->dir = di_left;
			turn = 1;
        }
        break;
    default:
        break;
    }
    if (obj->link->dir == di_left){
		if (obj->x == obj->link->x-2){
	        obj->dir = ReverseDir(obj->dir);
			turn = 1;
		}
	    set_emap_blockbox(Rbx(obj)+2, Lty(obj), Ltx(obj->link)-Ltx(obj)-2, 2);
		if (obj->dir == di_left){
			set_emap_box(Ltx(obj)-1, Lty(obj), 2, obj->height, eb_pierce);
			if (!turn){
				set_emap_box(Ltx(obj), Lty(obj)-1, Ltx(obj->link)-Ltx(obj)+obj->width, 1, eb_push_left);
			}
		}
		else {
			if (!turn){
				set_emap_box(Ltx(obj), Lty(obj)-1, Ltx(obj->link)-Ltx(obj)+obj->width, 1, eb_push_right);
			}
		}
	}
	else {
		if (obj->x == obj->link->x+6){
	        obj->dir = ReverseDir(obj->dir);
			turn = 1;
		}
	    set_emap_blockbox(Ltx(obj->link), Lty(obj), Ltx(obj)-Ltx(obj->link)-2, 2);
		if (obj->dir == di_right){
			set_emap_box(Rtx(obj)-1+1, Lty(obj), 2, obj->height, eb_pierce);
			if (!turn){
				set_emap_box(Ltx(obj->link), Lty(obj)-1, Ltx(obj)-Ltx(obj->link)+obj->width, 1, eb_push_right);
			}
		}
		else {
			if (!turn){
				set_emap_box(Ltx(obj->link), Lty(obj)-1, Ltx(obj)-Ltx(obj->link)+obj->width, 1, eb_push_left);
			}
		}
	}
    set_emap_blockbox(Ltx(obj), Lty(obj), obj->width, obj->height);

	return 0;
}


int act_lift(obj_t *obj){
    switch (State(obj)){
    case os_req_off:
        SetState(obj, os_off);
        break;
    case os_req_on:
        SetState(obj, os_on);
        break;
    case os_off:
        break;
    case os_on:
	    switch (obj->dir){
	    case di_up:
		    if (!check_emap_dir(eb_block_up, di_up, obj)){
			    obj->y --;
	            set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 1, eb_push_up);
	        }
	        else {
		        set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 4, eb_shock);
				obj->dir = di_down;
	            //            puts ("switch u->d");
		    }
			break;
	    case di_down:
		    if (!check_emap_dir(eb_block_down, di_down, obj)){
			    obj->y ++;
                //                printf("set eb_push_down\n");
	            set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 1, eb_push_down);
	            set_emap_box(Lbx(obj), Lby(obj)+1, obj->width, 1, eb_weight);
	        }
	        else {
	            set_emap_box(Ltx(obj), Rby(obj)+1, obj->width, 4, eb_shock);
				// ξ㳲ʪˤ֤Ĥä֤ǻߤޤäƤƤweightȯ
				set_emap_box(Lbx(obj), Lby(obj)+1, obj->width, 1, eb_weight);
			    obj->dir = di_up;
				//            puts ("switch u->d");
	        }
		    break;
	    case di_left:
		    //        printf("(%d, %d) %dx%d\n", obj->x, obj->y, obj->width, obj->height);
			if (!check_emap_dir(eb_block_left, di_left, obj)){
				obj->x --;
	            set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 1, eb_push_left);
			}
	        else {
		        set_emap_box(Ltx(obj)-1, Lty(obj), 4, obj->height, eb_shock);
				obj->dir = di_right;
	            //            puts ("switch l->r");
		    }
			break;
	    case di_right:
		    if (!check_emap_dir(eb_block_right, di_right, obj)){
			    obj->x ++;
	            set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 1, eb_push_right);
	        }         
		    else {
			    set_emap_box(Rbx(obj)+1, obj->y, 4, obj->height, eb_shock);
	            obj->dir = di_left;
		        //            puts ("switch r->l");
			}
	        break;
		default:
			break;
	    }
        break;
    default:
        break;
	}
    set_emap_blockbox(Ltx(obj), Lty(obj), obj->width, obj->height);
    return 0;
}

int act_mswitch(obj_t *obj){
	obj_state_t old_state = State(obj);

    if (check_emap_box_any(Ltx(obj), Rty(obj)-1, obj->width, 1, eb_weight)){
        if (obj->u.mswitch.is_reverse){
            SetState(obj, os_off);
            obj->motion = ii_mswitch_off;
        }
        else {
            SetState(obj, os_on);
            obj->motion = ii_mswitch_on;
        }
	}
	else {
        if (obj->u.mswitch.is_reverse){
            SetState(obj, os_on);
            obj->motion = ii_mswitch_on;
        }
        else {
            SetState(obj, os_off);
            obj->motion = ii_mswitch_off;
        }
	}
	if (State(obj) != old_state){
		if (obj->link){
			ReqState(obj->link, obj, State(obj) == os_on ? os_req_on : os_req_off);
		}
    }
	return 0;
}
                                                                                                                                                                                                                                               

void change_aswitch_weighted(obj_t *obj){
	if (State(obj) == os_nonweighted_off){
        SetState(obj, os_weighted_on);
        obj->motion = ii_aswitch_on;
        obj->y++;
        obj->height --;
        if (obj->link){
            ReqState(obj->link, obj, os_on);
        }
        printf("nwoff->won y=%d\n", obj->y);
    }
    else if (State(obj) == os_nonweighted_on){
        SetState(obj, os_weighted_off);
        obj->motion = ii_aswitch_off;
        obj->y--;
        obj->height ++;
        if (obj->link){
            ReqState(obj->link, obj, os_off);
        }
        printf("nwon->woff y=%d\n", obj->y);
    }
}
void change_aswitch_nonweighted(obj_t *obj){
    if (State(obj) == os_weighted_off){
        SetState(obj, os_nonweighted_off);
        printf("woff->nwoff y=%d\n", obj->y);
    }
    else if (State(obj) == os_weighted_on){
        SetState(obj, os_nonweighted_on_int);
        printf("won->nwon y=%d\n", obj->y);
    }
    else if (State(obj) == os_nonweighted_on_int){
        SetState(obj, os_nonweighted_on);
        printf("won->nwon y=%d\n", obj->y);
    }
}

// ϻONˤǤ褦ˡONˤ뤿νʬΥؿ
void on_aswitch(obj_t *obj){
        SetState(obj, os_nonweighted_on);
        obj->motion = ii_aswitch_on;
        obj->y++;
        obj->height --;
        if (obj->link){
            ReqState(obj->link, obj, os_on);
        }
}

int act_aswitch(obj_t *obj){
	int fl, fr;
	int x, y;
	obj_t *obj_l, *obj_r, *obj_m;
    //    printf("aswitch check y=%d\n", Lty(obj)-1);

    // !! hard_dynamicsoft_dynamicη̤next_emap˽񤤤Ƥ
    // äơcreature¸߾Ϥ1ƥåΤΤˤʤ

	y = Lty(obj)-1;
	fl = CheckEMap(Ltx(obj)-1, y, eb_weight) && CheckEMap(Ltx(obj)-1, Lty(obj), eb_exclude_up);
	fr = CheckEMap(Rtx(obj)+1, y, eb_weight) && CheckEMap(Rtx(obj)+1, Lty(obj), eb_exclude_up);
	obj_l = fl ? GetEMapOwner(Ltx(obj)-1, y, eb_weight) : NULL;
	obj_r = fr ? GetEMapOwner(Rtx(obj)+1, y, eb_weight) : NULL;

	if (State(obj) == os_req_on){
		on_aswitch(obj);
	}

	// åξʤȤɤäΩäƤԤ
	if (fl || fr){
		for (x = Ltx(obj) ; x <= Rtx(obj) ; x++){
			if (CheckEMap(x, y, eb_weight)){
				obj_m = GetEMapOwner(x, y, eb_weight);
				if (obj_m != obj_l && obj_m != obj_r){
					change_aswitch_weighted(obj);
					break;
				}
			}
		}

	}
	// åΤξƤΩäƤԤʤ
	else {
		// åΩäƤԤСΤå
		// OFFˤʤäȤ1ķ;夬ΤǥåϰϤι⤵2ˤƤ
	    if (check_emap_box_any(Ltx(obj), y, obj->width, 2, eb_weight)){
			change_aswitch_weighted(obj);
		}
		else {
			change_aswitch_nonweighted(obj);
		}
			
	}
    set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
	return 0;
}


int act_goal(obj_t *obj){
	switch (State(obj)){
    case os_none:
		break;
    case os_goal_req_open:
        if (obj->motion == ii_goal_closed){
            SetState(obj, os_goal_opening);
        }
        else {
            SetState(obj, os_goal_opened);
        }
        break;
	case os_goal_opening:
		IncCtr(obj);
		switch (Ctr(obj)){
		case 1:
			obj->motion = ii_goal_open1;
			break;
		case 2:
			obj->motion = ii_goal_open2;
			break;
		case 3:
			obj->motion = ii_goal_opened;
            SetState(obj, os_goal_opened);
			break;
		default:
			assert(0);
			break;
		}
		break;
	case os_goal_opened:
	    set_emap_box(obj->x, obj->y, obj->width, obj->height, eb_goal);
		break;
	case os_exit:
		IncCtr(obj);
        if (Ctr(obj) == CHEE_EXIT_COUNT){
            SetState(obj, os_goal_closing);
        }
		break;
	case os_goal_closing:
		IncCtr(obj);
		switch (Ctr(obj)){
		case 1:
			obj->motion = ii_goal_open2;
			break;
		case 2:
			obj->motion = ii_goal_open1;
			break;
		case 3:
			obj->motion = ii_goal_closed;
            SetState(obj, os_none);
			break;
		default:
			assert(0);
			break;
		}
	default:
		break;
	}
	return 0;
}

int act_fwall(obj_t *obj){
	switch (State(obj)){
    case os_none:
	    if (check_emap_obj_any_corner(obj, eb_shock)){
			// ɤblockingemap
			reset_emap_static_blockbox(obj->x, obj->y, obj->width, obj->height);
			SetState(obj, os_bang);
			obj->motion = ii_bang4_4;
			inc_kill_record(obj->type);
		}
		break;
    case os_bang:
		IncCtr(obj);
		if (Ctr(obj) == BANG_SPEED){
			return 1; // forall_obj¦ǺƤ餦
		}
		break;
	default:
		assert(0);
		break;
    }
    return 0;
}
int act_iwall(obj_t *obj){
	if (check_emap_obj_any_rim(obj, eb_flame)){
		IncCtr(obj);
		if(Ctr(obj) >= IWALL_LIFE){
			// ɤblockingemap
			reset_emap_static_blockbox(obj->x, obj->y, obj->width, obj->height);
			inc_kill_record(obj->type);
			return 1;
		}
	}
	return 0;
}
int act_barrier(obj_t *obj){
    switch (State(obj)){
    case os_req_off:
        SetState(obj, os_off);
        obj->motion = ii_none;
        break;
    case os_req_on:
        SetState(obj, os_on);
        obj->motion = ii_barrier0;
        break;
    case os_off:
        break;
    case os_on:
		if (obj->motion == ii_barrier3){
			obj->motion = ii_barrier0;
		}
		else {
			obj->motion ++;
		}
        set_emap_box(obj->x, obj->y, obj->width, obj->height, eb_flame);
        set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
        break;
    default:
        break;
    }
    return 0;
}
int act_flame(obj_t *obj){

	/*
	obj_t *new_flame;
	if (obj->u.flame.lenctr < FLAME_LEN){
		if (obj->dir == di_left){
//			if (!check_emap_box_any_rim(Ltx(obj)-(FLAME_WIDTH-1), Lty(obj), FLAME_WIDTH-1, FLAME_HEIGHT, eb_block_left) &&
			if (!check_emap_box_any_rim(Ltx(obj), Lty(obj), FLAME_WIDTH, FLAME_HEIGHT, eb_block_left) &&
				!CheckEMap(Ltx(obj)-1, Lty(obj), eb_flame))
			{
				new_flame = create_obj(ot_flame, Ltx(obj)-FLAME_WIDTH, Lty(obj), di_left);
				new_flame->u.flame.lenctr = obj->u.flame.lenctr+1;
			}
		}
		else {
//			if (!check_emap_box_any_rim(Rtx(obj)+1, Lty(obj), FLAME_WIDTH-1, FLAME_HEIGHT, eb_block_right) &&
			if (!check_emap_box_any_rim(Ltx(obj), Lty(obj), FLAME_WIDTH, FLAME_HEIGHT, eb_block_right) &&
				!CheckEMap(Rtx(obj)+1, Lty(obj), eb_flame))
			{
				new_flame = create_obj(ot_flame, Rtx(obj)+1, Lty(obj), di_right);
				new_flame->u.flame.lenctr = obj->u.flame.lenctr+1;
			}
		}
	}
*/
	IncCtr(obj);
	if (Ctr(obj) == FLAME_COUNT) return 1;
//	if (Ctr(obj) > 1){
		if (!check_emap_box_any_rim(Ltx(obj), Lty(obj), FLAME_WIDTH, FLAME_HEIGHT,
                                    eb_exclude_left)  &&
            !check_emap_box_any_rim(Ltx(obj), Lty(obj), FLAME_WIDTH, FLAME_HEIGHT,
                                    eb_exclude_right))
		{
            obj->x += (obj->dir == di_left) ? -FLAME_WIDTH+1 : FLAME_WIDTH-1;
		}
		else {
			return 1;
		}
//	}
        printf("flame emap(%d,%d) %dx%d -(%d, %d)\n", Ltx(obj), Lty(obj), obj->width, obj->height, Rtx(obj), Rty(obj));
        set_emap_box(obj->x, obj->y, obj->width, obj->height, eb_flame);
//	if (obj->x+obj->width-1 >= MAP_WIDTH){return 1;}

	return 0;


}
