#include <stdio.h>
#include <EzGraph.h>
#include "config.h"
#include "main.h"
#include "layout.h"
#include "select_stage.h"
#include "select_stage_draw.h"
#include "stage.h"

char *stage_names[] = {
	"walk_and_jump",
    "lift",
    "shortcut",
	"yaramashi",
    "sting",
    "hatopo",
    "ojigiri",
	"switch",
	"gagga",
	"freefall",
	"slope",
	"tunnel",
	"nya",
	"barrier",
	"sue",
	"fragile",
	"ojigiriwars",
	"flags",
	"gondin",
	"ojigiriwars2",

/*

	"test4",

	"heiankyo",
	"chobbit",
	"longjump",

	"gravity",
	"test",
	"test2",
    "test3",
	"pyramid",
	"slide",
	"ilikecats",
    "trap",
	*/
 };

#define STAGE_NUM (sizeof(stage_names)/sizeof(char*))

int stage_num = STAGE_NUM;
int line_num;

stage_thumb_t stage_thumbs[STAGE_NUM];
stage_thumb_t *current_stage_thumb = NULL;

static int cursor_x, cursor_y, line;


void select_stage_init(void){
    int i;

    for (i = 0 ; i < stage_num ; i++){
        printf("gen thumb:%s\n", get_stage_name(i));
        get_thumb(get_stage_name(i), &stage_thumbs[i]);
    }
    line = 0;
	cursor_x = 0;
	cursor_y = 0;
    line_num = (stage_num-1) / THUMB_COL_NUM + 1;
    printf("line_num = %d\n", line_num);
}

void setup_select_stage(void){
    EzSetKeyHandler(select_stage_keyh);
    EzSetKeyReleaseHandler(select_stage_key_releaseh);
    EzSetTimerHandler(select_stage_timerh, GAME_SPEED);
}

void select_stage_keyh(int key){
    printf("line=%d/%d, (%d,%d)", line, line_num, cursor_x, cursor_y);

    switch (key){
    case EZ_Left:
        cursor_x = (cursor_x + THUMB_COL_NUM - 1) % THUMB_COL_NUM;
        break;
    case EZ_Right:
        cursor_x = (cursor_x + 1) % THUMB_COL_NUM;
        break;
    case EZ_Up:
        if (cursor_y == 0){
            if (line > 0){
                line --;
            }
            else {
                line = line_num-THUMB_ROW_NUM;
                cursor_y = THUMB_ROW_NUM-1;
            }
        }
        else {
            cursor_y --;
        }
        break;
    case EZ_Down:
        if (cursor_y == THUMB_ROW_NUM-1){
            if (line < line_num-THUMB_ROW_NUM){
                line ++;
            }
            else {
                line = 0;
                cursor_y = 0;
            }
        }
        else {
            cursor_y ++;
        }
        break;
    default:
        break;
    }
    printf(" -> %d/%d, (%d,%d)\n", line, line_num, cursor_x, cursor_y);
}

void select_stage_key_releaseh(int key){
    int si;
    switch (key){
    case ' ':
    case EZ_Return:
        si = CursorPos2StageIx(cursor_x, cursor_y, line);
        if (si < stage_num){
			current_stage_thumb = &stage_thumbs[si];
            load_current_stage(get_stage_name(si), si+1);
            set_main_state(ms_game);
        }
        break;
    case 'r':
        si = CursorPos2StageIx(cursor_x, cursor_y, line);
        if (si < stage_num){
			current_stage_thumb = &stage_thumbs[si];
            //            load_current_stage(get_stage_name(sn), sn+1);
            load_current_stage_record(get_stage_name(si));
            set_main_state(ms_stage_info);
        }
        break;
    case 'q':
	case 27:
        set_main_state(ms_title);
		break;
    default:
        break;
    }
}

void select_stage_timerh(void){
    draw_select_stage(stage_thumbs, stage_num, cursor_x, cursor_y, line);
}

char *get_stage_name(int n){
    return stage_names[n];
}

unsigned int get_current_stage_no(void){
    return CursorPos2StageIx(cursor_x, cursor_y, line) + 1;
}

stage_thumb_t *get_current_thumb(void){
	return current_stage_thumb;
}
