#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <EzGraph.h>
#include "config.h"
#include "main.h"
#include "game.h"
#include "stage.h"
#include "select_stage.h"
#include "emap.h"
#include "obj.h"
#include "obj_draw.h"
#include "file.h"
#include "jstr.h"

#define BUF_SIZE 1024

static stage_t current_stage;
static char stage_dpath[BUF_SIZE];
static record_t current_stage_record;


static map_symbol_info_t map_symbol_info_list[] = {
    {' ', ot_blank},
    {'.', ot_blank},
    {'#', ot_wall},
    {'%', ot_fwall},
    {'*', ot_iwall},
    {'$', ot_barrier},
    {'/', ot_slope_l},
    {'\\', ot_slope_r},
    {'d', ot_stairs_l},
    {'b', ot_stairs_r},

	{'S', ot_sign},
    {'T', ot_sting},
    {'D', ot_goal},
    {'=', ot_lift},
    {'M', ot_mswitch},
    {'A', ot_aswitch},

	{'C', ot_chee},
    {'G', ot_gondin},
    {'H', ot_hatopo},
    {'Y', ot_yaarashi},
    {'O', ot_ojigiri},
    {'K', ot_gagga},
    {'N', ot_nya},
    {'U', ot_sue},
};
#define MAP_SYMBOL_INFO_NUM \
    (sizeof(map_symbol_info_list)/sizeof(map_symbol_info_t))

static obj_type_t symbol2type_table[128];

static char link_chrs[] = LINK_CHRS_STR;

static link_entry_t link_entries[LINK_NO_MAX];

static objective_t stage_objectives[OBJECTIVE_NUM];

void clear_stage(stage_t *stage);

void stage_init(char *argv0){
    int i;
    for (i = 0 ; i < MAP_SYMBOL_INFO_NUM ; i++){
        symbol2type_table[map_symbol_info_list[i].symbol]
            = map_symbol_info_list[i].type;
        printf("%c[%d] -> %d(%d)\n", map_symbol_info_list[i].symbol,
               map_symbol_info_list[i].symbol, map_symbol_info_list[i].type,
               symbol2type_table[map_symbol_info_list[i].symbol]);
    }

	// Visual StudioǤγȯϡ
	// ¹ԥե뤬Ȱۤʤǥ쥯ȥ˺Τǡ
	// ¹ԥե뤫Хǥ쥯ȥǤưʤ
	// ȥǥ쥯ȥϥ(ץȥե?)ξʤΤǡ
	// ȤꤢȤлˤƤ
#if defined(_MSC_VER) && defined(_DEBUG)
    sprintf(stage_dpath, "%s", STAGE_DIR);
#else
    sprintf(stage_dpath, "%s/%s", get_exe_dpath(argv0), STAGE_DIR);
#endif
}


void load_current_stage(char *name, int stage_no){
    load_stage(name, &current_stage);
	current_stage.no = stage_no;
    load_current_stage_record(name);
}


void clear_stage(stage_t *stage){
	int i, j;

    for (i = 0 ; i < STAGE_TITLE_ROW_NUM ; i++){
        stage->title[i][0] = fi_end;
    }
    for (i = 0 ; i < LINK_NO_MAX ; i++){
		link_entries[i].dist = NULL;
		link_entries[i].src = NULL;
        for (j = 0 ; j < STAGE_SIGN_ROW_NUM ; j++){
            stage->signs[i][j][0] = fi_end;
        }
    }
	stage->gondin_toeat = 1;
    for (i = 0 ; i < OBJECTIVE_NUM ; i++){
        stage_objectives[i].entry = re_invalid;
    }
}

int load_stage(char *stage_name, stage_t *stage){
    FILE *fp;
    char fpath[BUF_SIZE];
    char tline[BUF_SIZE];
    stage_command_t sc = sc_none;
    char *scp;
    char link_chr;
    int link_no = 0;
    int line_no = 0, map_row = 0, title_row = 0, sign_row = 0;
    static char *sc_str = SC_STR;
    int width;
    char name[BUF_SIZE];
    char opchr;
    unsigned int val;
    int objective_num = 0;
	int id;
    objective_info_t *objective_info;
	unsigned int v1, v2;

	clear_stage(stage);
	stage->name = stage_name;
    sprintf(fpath, "%s/%s.%s", stage_dpath,  stage_name, STAGE_EXT);
    printf("read %s\n", fpath);

    fp = fopen_bomcut(fpath, "r");
    if (fp == NULL){
        log_error("Cannot open stage file %s.\n", fpath);
        exit(1);
    }
    while (fgets(tline, BUF_SIZE, fp) != NULL){
        //        fprintf(stdout, "%d:%s", line_no, tline);
        line_no ++;
        if (tline[0] == '@'){
            scp = strchr(sc_str, tline[1]);
            if (scp == NULL){
                log_error("Unknown command @%c at line %d in %s\n",
                       tline[1], line_no, stage_name);
                exit(1);
            }
            sc = scp - sc_str+1;
            printf("stage command %d\n", sc);
            if (sc == sc_sign){
                if (sscanf(tline, "@sign %c", &link_chr) != 1){
                    log_error("Illegal sign line at line %d in %s\n",
                              line_no, stage_name);
                    exit(1);
                }
                link_no = chr2link_no(link_chr);
                sign_row = 0;
            }
			else if (sc == sc_gondin){
				if (sscanf(tline, "@gondin %d", &stage->gondin_toeat) != 1){
                    log_error("Illegal gondin line at line %d in %s\n",
                              line_no, stage_name);
                    exit(1);
                }
			}
			else if (sc == sc_version){
				if (sscanf(tline, "@version %d.%d", &v1, &v2) != 2){
                    log_error("Illegal version line at line %d in %s\n",
                              line_no, stage_name);
                    exit(1);
                }
				if (v1 < 256 && v2 < 256){
					stage->major_version = v1;
					stage->minor_version = v2;
				}
				else {
                    log_error("Illegal version value %d.%d at line %d in %s\n",
                              v1, v2, line_no, stage_name);
                    exit(1);
				}
			}
        }
        else {
            chop_line(tline);
			if (sc != sc_map && tline[0] == '#') continue;
            switch (sc){
            case sc_title:
                if (title_row >= STAGE_TITLE_ROW_NUM){
                    log_error("Too many title lines at line %d in %s\n",
                              line_no, stage_name);
                    exit(1);
                }
                width = str2fontseq(tline, stage->title[title_row]);
                //                printf("font num=%d\n", width);
                printf("stage_title %d %d %d %d\n", 
                       stage->title[map_row][0],
                       stage->title[map_row][1],
                       stage->title[map_row][2],
                       stage->title[map_row][3]
                       );
                title_row++;
                break;
            case sc_description:
                break;
            case sc_map:
                // ޥå׹ԤĹMAP_GRID_WIDTH*2+1ǤʤΥåɬ
//                printf("strncpy %d %d \n", strlen(tline), MAP_GRID_WIDTH*2+1);
                strncpy(stage->map[map_row], tline, MAP_GRID_WIDTH*2+1);
                //                                strcpy(stage->map[map_row], tline);
                map_row++;
                break;
            case sc_sign:
                if (sign_row == STAGE_SIGN_ROW_NUM){
                    log_error("Too many lines (limit: %d) in sign %d "
                              "at line %d in %s\n",
                              STAGE_SIGN_ROW_NUM, link_no, line_no, stage_name);
                    exit(1);
                }
                width = str2fontseq(tline, stage->signs[link_no][sign_row]);
                //                printf("font num=%d\n", width);
                sign_row ++;
				break;
            case sc_objective:
                if (sscanf(tline, "%s %c= %d", name, &opchr, &val) != 3){
                    log_error("Illegal objective line at line %d in %s\n",
                              line_no, stage_name);
                    exit(1);
                }
                printf("mission:%s:%d %c %d\n",
                       tline, objective_name2id(name), opchr, val);
                if (objective_num >= OBJECTIVE_NUM){
                    log_error("Too many objectives at line %d in %s\n",
                              line_no, stage_name);
                    exit(1);
                }
				id = objective_name2id(name);
				if (id < 0){
					log_error("Unknown objective '%s' at line %d in %s\n",
                              name, line_no, stage_name);
                    exit(1);
				}
                objective_info = get_objective_info(id);
				stage_objectives[objective_num].entry
                    = objective_info->entry;
				stage_objectives[objective_num].icon
                    = objective_info->icon;
                stage_objectives[objective_num].incdec
                    = objective_info->incdec;
                stage_objectives[objective_num].description_fontseq
                    = objective_info->description_fontseq;
                switch (opchr){
                case '>':
                    stage_objectives[objective_num].op = oo_ge;
                    break;
                case '<':
                    stage_objectives[objective_num].op = oo_le;
                    break;
                case '=':
                    stage_objectives[objective_num].op = oo_eq;
                    break;
                default:
                    log_error("Unknown objective operator %c "
                              "at line %d in %s\n",
                              opchr, line_no, stage_name);
                    exit(1);
                    break;
                }
                stage_objectives[objective_num].val = val;
                objective_num ++;
                break;
            default:
                assert(0);
                exit(1);
            }
        }
    }
    /*
    for (i = 0 ; i < MAP_GRID_HEIGHT ; i++){
        fgets(stage->map[i], MAP_GRID_WIDTH*2+2, fp);
        fprintf(stdout, "%d:%s", i, stage->map[i]);
    }
    */
    fclose(fp);
    return -1;
}

font_id_t *get_current_stage_title(int row){
    return current_stage.title[row];
}
char *get_current_stage_name(void){
    return current_stage.name;
}
int get_current_stage_sign_no(void){
	return current_stage.sign_no;
}
void set_current_stage_sign_no(int no){
	current_stage.sign_no = no;
}

font_id_t *get_current_stage_sign(int row){
	assert(current_stage.sign_no != INVALID_LINK_NO);
    return current_stage.signs[current_stage.sign_no][row];
}
int get_current_stage_gondin_toeat(void){
	return current_stage.gondin_toeat;
}
int get_current_stage_gondin_total(void){
	return current_stage.gondin_total;
}

//unsigned int get_current_stage_no(void){
//	return current_stage.no;
//}

obj_type_t symbol2type(unsigned char symbol){
    //    printf("<<%d:%d>>", symbol, symbol2type_table[symbol]);
    return symbol2type_table[symbol];
}

void set_thumb_stars(stage_thumb_t *thumb){
	int i;
	objective_t *objective;

	for (i = 0 ; i < OBJECTIVE_NUM ; i++){
		objective = get_stage_objective(i);
		if (objective->entry == re_invalid){
			thumb->stars_fontseq[i] = fi_space;
		}
		else if (objective->op == oo_ge){
			if (CurrentStageRecordMax(objective->entry) >= objective->val){
				thumb->stars_fontseq[i] = fi_star_on;
			}
			else {
				thumb->stars_fontseq[i] = fi_star_off;
			}
		}
		else {
			if (CurrentStageRecordMin(objective->entry) <= objective->val){
				thumb->stars_fontseq[i] = fi_star_on;
			}
			else {
				thumb->stars_fontseq[i] = fi_star_off;
			}
		}
	}
	thumb->stars_fontseq[OBJECTIVE_NUM] = fi_end;
}

void get_thumb(char *stage_name, stage_thumb_t *thumb){
    int x, y, i;
    obj_type_t type;
    
    load_current_stage(stage_name, 0);
    for (i = 0 ; i < STAGE_TITLE_ROW_NUM ; i++){
        fontseq_cpy(thumb->title[i], current_stage.title[i]);
    }
	thumb->top_score = CurrentStageRecordMax(re_score);
	set_thumb_stars(thumb);

	
	
	
	for (y = 0 ; y < MAP_GRID_HEIGHT ; y++){
        for (x = 0 ; x < MAP_GRID_WIDTH ; x++){
            type = symbol2type(GetMapSymbol(x, y));
            switch(type){
            case ot_wall:
            case ot_fwall:
                SetThumbColor(x, y, 2);
				break;
            case ot_iwall:
                SetThumbColor(x, y, 7);
				break;
            case ot_barrier:
                SetThumbColor(x, y, 5);
				break;
            case ot_lift:
            case ot_mswitch:
            case ot_aswitch:
                SetThumbColor(x, y, 4);
				break;
            case ot_sting:
                SetThumbColor(x, y, 3);
				break;
            case ot_chee:
				set_thumb_color_box(x, y, 2, 2, 5, thumb);
				break;
            case ot_goal:
				set_thumb_color_box(x, y, 2, 2, 5, thumb);
				break;
            case ot_gondin:
				set_thumb_color_box(x, y, 2, 2, 6, thumb);
				break;
            case ot_nya:
				set_thumb_color_box(x, y, 2, 4, 1, thumb);
				break;
            case ot_ojigiri:
				set_thumb_color_box(x, y, 2, 4, 1, thumb);
				break;
            case ot_sue:
				set_thumb_color_box(x, y, 3, 2, 1, thumb);
				break;
            case ot_yaarashi:
				set_thumb_color_box(x, y, 2, 1, 1, thumb);
				break;
            case ot_hatopo:
            case ot_gagga:
				set_thumb_color_box(x, y, 2, 2, 1, thumb);
				break;
            default:
                break;
            }
        }
    }
}

void set_thumb_color_box(int ltx, int lty, int width, int height, int color, stage_thumb_t *thumb){
	int x, y;
	for (y = lty ; y < lty+height ; y++){
		for (x = ltx ; x < ltx+width ; x++){
			SetThumbColor(x, y, color);
		}
	}
}


dir_t get_map_objdir(stage_t *stage, int x, int y){
    switch (GetMapOption(x, y)){
    case '<':
        return di_left;
    case '>':
        return di_right;
    case '^':
        return di_up;
    case 'v':
        return di_down;
    case ' ':
        return di_none;
    default:
        log_error("Unknown map option '%c' at (%d, %d).\n",
                  GetMapOption(x, y), x, y);
        exit(1);
    }
	return di_none;
}

int chr2link_no(char c){
    char *p;
    p = strchr(link_chrs, c);
    if (p == NULL) return INVALID_LINK_NO;
    return p - link_chrs;
}

int get_map_link(stage_t *stage, int x, int y){
    char c;
    int no;

    c = GetMapOption(x, y);
    no = chr2link_no(c);
	if (no == INVALID_LINK_NO){
        c = GetMapOption(x+1, y);
        no = chr2link_no(c);
	}
    return no;
}

int chr2onoff(char c){
   if (c == '+'){
		return 1;
	}
	else if (c == '-'){
		return -1;
	}
	else {
		return 0;
	}
}

int get_onoff(stage_t *stage, int x, int y){
	char c;
	int onoff;
	c = GetMapOption(x, y);
	onoff = chr2onoff(c);
	if (onoff == 0){
        c = GetMapOption(x+1, y);
        onoff = chr2onoff(c);
	}
    return onoff;
}

void register_link_src(obj_t *src, int link_no){
    assert(link_no != INVALID_LINK_NO);
    printf("register link %d src %p\n", link_no, src);
    link_entries[link_no].src = src;
}
void register_link_dist(obj_t *dist, int link_no){
    assert(link_no != INVALID_LINK_NO);
    printf("register link %d dist %p\n", link_no, dist);
    link_entries[link_no].dist = dist;
}

void apply_all_links(void){
    int i;
    for (i = 0 ; i < LINK_NO_MAX ; i++){
        if (link_entries[i].src != NULL &&
            link_entries[i].dist != NULL)
        {
            link_entries[i].src->link = link_entries[i].dist;
            link_entries[i].dist->link = link_entries[i].src;
            printf("set link %d (%p->%p)\n",
                   i, link_entries[i].src, link_entries[i].dist);
			switch (link_entries[i].src->type){
			case ot_aswitch:
				if (State(link_entries[i].src) == os_nonweighted_on){
					SetState(link_entries[i].dist, os_on);
				}
				else {
					SetState(link_entries[i].dist, os_off);
				}
				break;
			case ot_mswitch:
				if (link_entries[i].src->u.mswitch.is_reverse){
					SetState(link_entries[i].dist, os_on);
				}
				else {
					SetState(link_entries[i].dist, os_off);
				}
				break;
			default:
				assert(0);
				break;
			}
        }
    }
}

void setup_stage(void){
    int x, rx, y, ry, len;
    dir_t dir = di_none;
    obj_t *obj;
    obj_type_t type;
    int link_no, onoff;

    setup_obj_list();
	set_chee(NULL);
    current_stage.gondin_total = 0;
	current_stage.sign_no = INVALID_LINK_NO;
	clear_emap();

    for (y = 0 ; y < MAP_GRID_HEIGHT ; y++){
        for (x = 0 ; x < MAP_GRID_WIDTH ; x++){
            type = symbol2type(GetMapSymbol(x, y));
            //                        printf("(%2d, %2d) [%c](%d) %d\n", x, y, GetMapSymbol(x, y), GetMapSymbol(x, y), type);
            switch(type){
			case ot_blank:
				break;
            case ot_chee:
                if (get_chee()){
                    log_error("Multiple CheeChee.\n");
                    exit(1);
                }
                dir = get_map_objdir(&current_stage, x, y);
                if (dir == di_none) dir = di_left;
                create_obj_grid(type, x, y, dir);
                break;
            case ot_hatopo:
            case ot_nya:
			case ot_yaarashi:
			case ot_ojigiri:
            case ot_sue:
                dir = get_map_objdir(&current_stage, x, y);
                if (dir == di_none) dir = di_left;
                create_obj_grid(type, x, y, dir);
                break;
            case ot_wall:
            case ot_fwall:
            case ot_iwall:
			case ot_goal:
            case ot_slope_l:
            case ot_slope_r:
            case ot_stairs_l:
            case ot_stairs_r:
            case ot_gagga:
                create_obj_grid(type, x, y, di_none);
                break;
            case ot_gondin:
                current_stage.gondin_total ++;
                create_obj_grid(type, x, y, di_none);
                break;
            case ot_sting:
                dir = get_map_objdir(&current_stage, x, y);
                if (dir == di_none) dir = di_left;
                create_obj_grid(type, x, y, dir);
                break;
			case ot_sign:
                link_no = get_map_link(&current_stage, x, y);
                create_sign_grid(type, x, y, link_no);
				break;
            case ot_lift:
                dir = get_map_objdir(&current_stage, x, y);
                link_no = get_map_link(&current_stage, x, y);
                if (dir == di_none) dir = di_left;
                obj = create_obj_grid(ot_lift, x, y, dir);
                if (link_no != INVALID_LINK_NO){
                    register_link_dist(obj, link_no);
                }
                len = 1;
                rx = x+1;
                while (symbol2type(GetMapSymbol(rx, y)) == ot_lift){
                    len++;
                    rx++;
                }
                set_grid_width_dynamic(obj, len);
                x = rx-1;
                break;
            case ot_barrier:
                // Хꥢʤ顢Ǥ˽ѤߤΥХꥢΰʤΤǥѥ
                if (symbol2type(GetMapSymbol(x, y-1)) == ot_barrier) break;

                link_no = get_map_link(&current_stage, x, y);
                obj = create_obj_grid(ot_barrier, x, y, dir);
                if (link_no != INVALID_LINK_NO){
                    register_link_dist(obj, link_no);
                }
                len = 1;
                rx = x+1;
                while (symbol2type(GetMapSymbol(rx, y)) == ot_barrier){
                    len++;
                    rx++;
                }
                set_grid_width_dynamic(obj, len);
                x = rx-1;
                len = 1;
                ry = y+1;
                while (symbol2type(GetMapSymbol(x, ry)) == ot_barrier){
                    len++;
                    ry++;
                }
                set_grid_height_dynamic(obj, len);
                break;
            case ot_mswitch:
            case ot_aswitch:
                obj = create_obj_grid(type, x, y, di_none);
                link_no = get_map_link(&current_stage, x, y);
                if (link_no != INVALID_LINK_NO){
                    register_link_src(obj, link_no);
                }
                onoff = get_onoff(&current_stage, x, y);
                if (onoff == 1){
                    if (type == ot_aswitch){
						on_aswitch(obj);
//                        SetState(obj, os_req_on);
                    }
                    else {
                        obj->u.mswitch.is_reverse = 1;
						SetState(obj, os_on);
                        obj->motion = ii_mswitch_on;
                    }
                }
                len = 1;
                rx = x+1;
                while (symbol2type(GetMapSymbol(rx, y)) == type){
                    len++;
                    rx++;
                }
                if (type == ot_mswitch){
                    set_grid_width_static(obj, len);
                }
                else {
                    set_grid_width_dynamic(obj, len);
                }
                x = rx-1;
                break;
            default:
                break;
            }
        }
    }
    if (get_chee() == NULL){
        log_error("No CheeChee.\n");
        exit(1);
    }

    apply_all_links();
    setup_emap();
}

// βԤ
void chop_line(char *tline){
    int last = strlen(tline)-1;
    if (tline[last] == '\n'){
        tline[last] = '\0';
    }
}


void update_stage_record(game_t *game){
	stage_thumb_t *thumb;
    int i;

	print_stage_record();
    for (i = re_begin ; i < re_end ; i++){
        if (current_stage_record.max[i] < game->record[i]){
            current_stage_record.max[i] = game->record[i];
        }
        if (current_stage_record.min[i] > game->record[i]){
            current_stage_record.min[i] = game->record[i];
        }
        current_stage_record.sum[i] += game->record[i];
    }
	print_stage_record();
	thumb = get_current_thumb();
	thumb->top_score = current_stage_record.max[re_score];
	set_thumb_stars(thumb);
}


int load_stage_record(char *name, record_t *stage_record, unsigned char stage_major_version, unsigned char stage_minor_version){
    FILE *fp;
	char path[BUF_SIZE];
    unsigned char file_major_version = 0, file_minor_version = 0;
	unsigned char major_version = MAJOR_VERSION, minor_version = MINOR_VERSION;
	unsigned char file_stage_major_version = 0, file_stage_minor_version = 0;

	sprintf(path, "%s/%s.%s", STAGE_RECORD_DIR, name, STAGE_RECORD_EXT);
	if ((fp = open_data(path, "rb",
                        APPLICATION_ID, APPLICATION_ID_SIZE,
                        &file_major_version, &file_minor_version)) == NULL)
    {
		// ̤ץ쥤ʤե뤬ʤΤʤΤǤٹ𤷤ʤ
		//        log_warn("Cannot open stage record file %s\n", path);
        return 0;
    }
    fread(&file_stage_major_version, sizeof(unsigned char), 1, fp);
    fread(&file_stage_minor_version, sizeof(unsigned char), 1, fp);
    // Сʤɤ߹
    if (GetVersion(file_major_version, file_minor_version) <
        GetVersion(major_version, minor_version))
    {
        log_warn("Cannot read record file '%s' of older game(Ver%d.%2d)... renamed.\n",
                 path, file_major_version, file_minor_version);
        fclose(fp);
        rename_data(path);
        return 0;
    }
    if (GetVersion(file_stage_major_version, file_stage_minor_version) <
        GetVersion(stage_major_version, stage_minor_version))
    {
        log_warn("Cannot read record file '%s' of older stage(Ver%d.%2d)... renamed.\n",
                 path, file_stage_major_version, file_stage_minor_version);
        fclose(fp);
        rename_data(path);
        return 0;
    }
	printf("load %d mins\n", re_end);
    fread_uint(fp, stage_record->min, re_end);
	printf("loaded min:%d %d %d\n", stage_record->min[0],stage_record->min[1],stage_record->min[2]); 
	printf("load %d maxs\n", re_end);
    fread_uint(fp, stage_record->max, re_end);
	printf("load %d sums\n", re_end);
    fread_uint(fp, stage_record->sum, re_end);
    fclose(fp);
    return 1;
}

void load_current_stage_record(char *name){
    int i;
	if (!load_stage_record(name, &current_stage_record, current_stage.major_version, current_stage.minor_version)){
		memset(&current_stage_record, 0, sizeof(current_stage_record));
        for (i = 0 ; i < re_end ; i++){
            current_stage_record.min[i] = MINMAX_MAX;
        }
	}
	print_stage_record();
}


int save_stage_record(char *name, record_t *stage_record, unsigned char stage_major_version, unsigned char stage_minor_version){
	FILE *fp;
	char path[BUF_SIZE];

	sprintf(path, "%s/%s", get_save_dpath(), STAGE_RECORD_DIR);
// 	STAGE_RECORD_DIRʤк
    if (check_dir(path) == 0){
        log_warn("Cannot find '%s'.\n", path);
        if (make_dir(path) == 0){
            log_warn("Created '%s'.\n", path);
        }
        else {
            log_warn("Creating '%s' failed.\n", path);
            return 0;
        }
	}
	sprintf(path, "%s/%s.%s", STAGE_RECORD_DIR, name, STAGE_RECORD_EXT);
    if ((fp = open_data(path, "wb",
                        APPLICATION_ID, APPLICATION_ID_SIZE,
                        &major_version, &minor_version)) == NULL)
    {
		log_warn("Failed saving stage record %s.\n", path);
        return 0;
    }
    fwrite(&stage_major_version, sizeof(unsigned char), 1, fp);
    fwrite(&stage_minor_version, sizeof(unsigned char), 1, fp);
    fwrite_uint(fp, stage_record->min, re_end);
	printf("saved min:%d %d %d\n", stage_record->min[0],stage_record->min[1],stage_record->min[2]); 
    fwrite_uint(fp, stage_record->max, re_end);
    fwrite_uint(fp, stage_record->sum, re_end);
    /*
    fwrite_uint(fp, &current_stage_record.top_score, 1);
	fwrite_uint(fp, &current_stage_record.top_time, 1);
	fwrite(&current_stage_record.top_gondin_eaten, 1, 1, fp);
    */
	fclose(fp);
	return 1;
}

void save_current_stage_record(char *name){
	save_stage_record(name, &current_stage_record, current_stage.major_version, current_stage.minor_version);
}

unsigned int get_current_stage_record_min(record_entry_t entry){
    return current_stage_record.min[entry];
}
unsigned int get_current_stage_record_max(record_entry_t entry){
    return current_stage_record.max[entry];
}
unsigned int get_current_stage_record_sum(record_entry_t entry){
    return current_stage_record.sum[entry];
}

void inc_current_stage_record(record_entry_t entry){
    current_stage_record.sum[entry] ++;
}

objective_t *get_stage_objective(int no){
    return &stage_objectives[no];
}

/*
font_id_t objective_id2font(int id){
    return objective_infos[id].font;
}
*/

#ifdef DEBUG
void print_stage_record(void){
    printf("score:%d-%d %d\n",
           CurrentStageRecordMin(re_score), 
           CurrentStageRecordMax(re_score),
           CurrentStageRecordSum(re_score));
    printf("time:%d-%d %d\n",
           CurrentStageRecordMin(re_time), 
           CurrentStageRecordMax(re_time),
           CurrentStageRecordSum(re_time));
    printf("time:%d-%d %d\n",
           CurrentStageRecordMin(re_gondin_eaten), 
           CurrentStageRecordMax(re_gondin_eaten),
           CurrentStageRecordSum(re_gondin_eaten));
}
#endif
