#ifndef STAGE_H
#define STAGE_H
#include "config.h"
#include "layout.h"
#include "obj.h"
#include "font.h"
#include "game.h"
#include "record.h"

typedef struct {
    unsigned char symbol;
    obj_type_t type;
} map_symbol_info_t;

#define LINK_NO_MAX (10+26-1)
#define LINK_CHRS_STR "0123456789acefghijklmnopqrstuwxyz"
#define SC_STR "tdmosgv"
typedef enum {
    sc_none, sc_title, sc_description, sc_map, sc_objective, sc_sign, sc_gondin, sc_version,
} stage_command_t;

typedef struct {
	char *name;
	unsigned char major_version;
	unsigned char minor_version;
	unsigned int no;
	unsigned int gondin_toeat;
	unsigned int gondin_total;
    font_id_t title[STAGE_TITLE_ROW_NUM][STAGE_TITLE_WIDTH+1];
	int sign_no;
    font_id_t signs[LINK_NO_MAX][STAGE_SIGN_ROW_NUM][STAGE_SIGN_WIDTH+1];
    //    font_id_t description[STAGE_DESCRIPTION_ROW_NUM][STAGE_DESCRIPTION_WIDTH+1];
    char map[MAP_GRID_HEIGHT][MAP_GRID_WIDTH*2+2];
} stage_t;

#define CurrentStageRecordSum(re) (RecordSum(current_stage_record, re))
#define CurrentStageRecordMin(re) (RecordMin(current_stage_record, re))
#define CurrentStageRecordMax(re) (RecordMax(current_stage_record, re))
#define CountRecord(re) (inc_current_stage_record(re))


typedef struct {
	font_id_t stars_fontseq[4];
    font_id_t title[STAGE_TITLE_ROW_NUM][STAGE_TITLE_WIDTH+1];
	unsigned int top_score;
    int map[MAP_GRID_HEIGHT][MAP_GRID_WIDTH];
} stage_thumb_t;

typedef struct {
    obj_t *src;
    obj_t *dist;
} link_entry_t;

#define INVALID_LINK_NO (-1)

#define GetMapSymbol(x, y) (current_stage.map[y][(x)*2])
#define GetMapOption(x, y) (current_stage.map[y][(x)*2+1])
#define SetThumbColor(x, y, c) (thumb->map[y][x] = c)
#define GetThumbColor(x, y) (thumb->map[y][x])

void stage_init(char *argv0);
void load_current_stage(char *name, int stage_no);
int load_stage(char *map_name, stage_t *stage);
font_id_t *get_current_stage_sign(int row);
int get_current_stage_sign_no(void);
void set_current_stage_sign_no(int no);
font_id_t *get_current_stage_title(int row);
char *get_current_stage_name(void);
unsigned int get_current_stage_no(void);
int get_current_stage_gondin_toeat(void);
int get_current_stage_gondin_total(void);
obj_type_t symbol2type(unsigned char symbol);
void get_thumb(char *filename, stage_thumb_t *thumb);
void set_thumb_color_box(int ltx, int lty, int width, int height, int color, stage_thumb_t *thumb);
dir_t get_map_objdir(stage_t *stage, int x, int y);
void setup_stage(void);
obj_t *get_chee(void);
void chop_line(char *tline);
int chr2link_no(char c);
void update_stage_record(game_t *game);
int load_stage_record(char *name, record_t *stage_record, unsigned char stage_major_version, unsigned char stage_minor_version);
void load_current_stage_record(char *name);
int save_stage_record(char *name, record_t *stage_record, unsigned char stage_major_version, unsigned char stage_minor_version);
void save_current_stage_record(char *name);
unsigned int get_current_stage_record_min(record_entry_t entry);
unsigned int get_current_stage_record_max(record_entry_t entry);
unsigned int get_current_stage_record_sum(record_entry_t entry);
void inc_current_stage_record(record_entry_t entry);

int objective_name2id(char *name);
record_entry_t objective_id2entry(int id);
objective_t *get_stage_objective(int no);
//font_id_t objective_id2font(int id);

#ifdef DEBUG
void print_stage_record(void);
#endif


#endif

